@@ -61,19 +61,19 @@
  *
  */
 static GdkPoint 
-rotate_point(GdkPoint point, int angle)
+rotate_point(GdkPoint point, double angle)
 {
     double sint, cost;
     GdkPoint returned;
     
-    if (angle == 0)
+    if (angle == 0.0)
 	return point;
 
-    sint = sin(-(double)angle * M_PI / 180.0);
-    cost = cos(-(double)angle * M_PI / 180.0);
+    sint = sin(DEG2RAD(-angle));
+    cost = cos(DEG2RAD(-angle));
     
-    returned.x = (int)round(cost * (double)point.x - sint * (double)point.y);
-    returned.y = (int)round(sint * (double)point.x + cost * (double)point.y);
+    returned.x = lround(cost*point.x - sint*point.y);
+    returned.y = lround(sint*point.x + cost*point.y);
     
     return returned;
 }
@@ -148,7 +148,7 @@ gerbv_gdk_draw_prim4(GdkPixmap *pixmap, GdkGC *gc, double *p,
 
     /* Include start point */
     nuf_points = (int)p[nuf_points_idx] + 1;
-    points = (GdkPoint *)g_malloc(sizeof(GdkPoint) * nuf_points);
+    points = g_new(GdkPoint, nuf_points);
     if (!points) {
 	g_free(points);
 	return;
@@ -207,7 +207,7 @@ gerbv_gdk_draw_prim5(GdkPixmap *pixmap, GdkGC *gc, double *p,
     GdkColor color;
 
     nuf_vertices = (int)p[nuf_vertices_idx];
-    points = (GdkPoint *)g_malloc(sizeof(GdkPoint) * nuf_vertices);
+    points = g_new(GdkPoint, nuf_vertices);
     if (!points) {
 	g_free(points);
 	return;
@@ -222,14 +222,12 @@ gerbv_gdk_draw_prim5(GdkPixmap *pixmap, GdkGC *gc, double *p,
     }
 
     tick = 2 * M_PI / (double)nuf_vertices;
-    rotation = -p[rotation_idx] * M_PI / 180.0;
+    rotation = DEG2RAD(-p[rotation_idx]);
     radius = p[diameter_idx] / 2.0;
     for (i = 0; i < nuf_vertices; i++) {
 	vertex =  tick * (double)i + rotation;
-	points[i].x = (int)round(scale * radius * cos(vertex)) + x +
-	    p[center_x_idx];
-	points[i].y = (int)round(scale * radius * sin(vertex)) + y +
-	    p[center_y_idx];
+	points[i].x = (int)round(scale * (radius * cos(vertex) + p[center_x_idx])) + x;
+	points[i].y = (int)round(scale * (radius * sin(vertex) - p[center_y_idx])) + y;
     }
 
     gdk_draw_polygon(pixmap, local_gc, 1, points, nuf_vertices);
@@ -484,9 +482,11 @@ gerbv_gdk_draw_prim21(GdkPixmap *pixmap, GdkGC *gc, double *p,
     points[3].y = half_height;
 
     for (i = 0; i < nuf_points; i++) {
+	points[i].x += (int)(p[exp_x_idx] * scale);
+	points[i].y -= (int)(p[exp_y_idx] * scale);
 	points[i] = rotate_point(points[i], p[rotation_idx]);
-	points[i].x += (x + (int)(p[exp_x_idx] * scale));
-	points[i].y += (y - (int)(p[exp_y_idx] * scale));
+	points[i].x += x;
+	points[i].y += y;
     }
 
     gdk_gc_copy(local_gc, gc);
@@ -541,7 +541,7 @@ gerbv_gdk_draw_prim22(GdkPixmap *pixmap, GdkGC *gc, double *p,
 			     * scale);
 
     for (i = 0; i < nuf_points; i++) {
-	points[i] = rotate_point(points[i], p[rotation_idx]);
+	points[i] = rotate_point(points[i], -p[rotation_idx]);
 	points[i].x = x + points[i].x;
 	points[i].y = y - points[i].y;
     }
@@ -561,57 +561,40 @@ gerbv_gdk_draw_prim22(GdkPixmap *pixmap, GdkGC *gc, double *p,
     return;
 } /* gerbv_gdk_draw_prim22 */
 
+/* Keep this array in order and without holes */
+static void (*dgk_draw_amacro_funcs[])(GdkPixmap *, GdkGC *, double *, 
+		     double, gint, gint) = {
+	[GERBV_APTYPE_MACRO_CIRCLE]  = &gerbv_gdk_draw_prim1,
+	[GERBV_APTYPE_MACRO_OUTLINE] = &gerbv_gdk_draw_prim4,
+	[GERBV_APTYPE_MACRO_POLYGON] = &gerbv_gdk_draw_prim5,
+	[GERBV_APTYPE_MACRO_MOIRE]   = &gerbv_gdk_draw_prim6,
+	[GERBV_APTYPE_MACRO_THERMAL] = &gerbv_gdk_draw_prim7,
+	[GERBV_APTYPE_MACRO_LINE20]  = &gerbv_gdk_draw_prim20,
+	[GERBV_APTYPE_MACRO_LINE21]  = &gerbv_gdk_draw_prim21,
+	[GERBV_APTYPE_MACRO_LINE22]  = &gerbv_gdk_draw_prim22,
+};
 
-static void 
+static inline void 
 gerbv_gdk_draw_amacro(GdkPixmap *pixmap, GdkGC *gc, 
 		      gerbv_simplified_amacro_t *s, double scale, 
 		      gint x, gint y)
 {
-    gerbv_simplified_amacro_t *ls = s;
+	dprintf("%s(): drawing simplified aperture macros:\n", __func__);
 
-    dprintf(_("Drawing simplified aperture macros:\n"));
-    while (ls != NULL) {
+	while (s != NULL) {
+		if (s->type >= GERBV_APTYPE_MACRO_CIRCLE
+		 && s->type <= GERBV_APTYPE_MACRO_LINE22) {
+			dgk_draw_amacro_funcs[s->type](pixmap, gc,
+					s->parameter, scale, x, y);
+			dprintf("  %s\n", gerbv_aperture_type_name(s->type));
+		} else {
+			GERB_FATAL_ERROR(
+				_("Unknown simplified aperture macro type %d"),
+				s->type);
+		}
 
-	switch (ls->type) {
-	case GERBV_APTYPE_MACRO_CIRCLE:
-	    gerbv_gdk_draw_prim1(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Circle\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_OUTLINE:
-	    gerbv_gdk_draw_prim4(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Outline\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_POLYGON:
-	    gerbv_gdk_draw_prim5(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Polygon\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_MOIRE:
-	    gerbv_gdk_draw_prim6(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Moire\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_THERMAL:
-	    gerbv_gdk_draw_prim7(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Thermal\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_LINE20:
-	    gerbv_gdk_draw_prim20(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Line 20\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_LINE21:
-	    gerbv_gdk_draw_prim21(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Line 21\n"));
-	    break;
-	case GERBV_APTYPE_MACRO_LINE22:
-	    gerbv_gdk_draw_prim22(pixmap, gc, ls->parameter, scale, x, y);
-	    dprintf(_("  Line 22\n"));
-	    break;
-	default:
-	    GERB_FATAL_ERROR(_("Unknown simplified aperture macro"));
+		s = s->next;
 	}
-
-	ls = ls->next;
-    }
-
 } /* gerbv_gdk_draw_amacro */
 
 
@@ -635,17 +618,32 @@ gerbv_gdk_draw_circle(GdkPixmap *pixmap, GdkGC *gc,
 /*
  * Draws a rectangle _centered_ at x,y with sides x_side, y_side
  */
-static void 
-gerbv_gdk_draw_rectangle(GdkPixmap *pixmap, GdkGC *gc, 
-		     gint filled, gint x, gint y, gint x_side, gint y_side)
+static void
+gerbv_gdk_draw_rectangle(GdkPixmap *pixmap, GdkGC *gc,
+		int filled, gint x, gint y, gint x_side, gint y_side,
+		double angle_deg)
 {
-    
-    gint real_x = x - x_side / 2;
-    gint real_y = y - y_side / 2;
-    
-    gdk_draw_rectangle(pixmap, gc, filled, real_x, real_y, x_side, y_side);
-    
-    return;
+	int i;
+	GdkPoint points[4];
+
+	points[0].x = -(x_side >> 1);
+	points[0].y = -(y_side >> 1);
+	points[1].x = x_side >> 1;
+	points[1].y = points[0].y;
+	points[2].x = points[1].x;
+	points[2].y = y_side >> 1;
+	points[3].x = points[0].x;
+	points[3].y = points[2].y;
+
+	for (i = 0; i < 4; i++) {
+		points[i] = rotate_point(points[i], angle_deg);
+		points[i].x += x;
+		points[i].y += y;
+	}
+
+	gdk_draw_polygon(pixmap, gc, filled, points, 4);
+
+	return;
 } /* gerbv_gdk_draw_rectangle */
 
 
@@ -653,31 +651,46 @@ gerbv_gdk_draw_rectangle(GdkPixmap *pixmap, GdkGC *gc,
  * Draws an oval _centered_ at x,y with x axis x_axis and y axis y_axis
  */ 
 static void
-gerbv_gdk_draw_oval(GdkPixmap *pixmap, GdkGC *gc, 
-		gint filled, gint x, gint y, gint x_axis, gint y_axis)
+gerbv_gdk_draw_oval(GdkPixmap *pixmap, GdkGC *gc,
+		int filled, gint x, gint y, gint x_axis, gint y_axis,
+		double angle_deg)
 {
-    gint delta = 0;
-    GdkGC *local_gc = gdk_gc_new(pixmap);
+	gint width;
+	GdkPoint points[2];
+	GdkGC *local_gc = gdk_gc_new(pixmap);
 
-    gdk_gc_copy(local_gc, gc);
+	gdk_gc_copy(local_gc, gc);
 
-    if (x_axis > y_axis) {
-	/* Draw in x axis */
-	delta = x_axis / 2 - y_axis / 2;
-	gdk_gc_set_line_attributes(local_gc, y_axis, 
-				   GDK_LINE_SOLID, 
-				   GDK_CAP_ROUND, 
-				   GDK_JOIN_MITER);
-	gdk_draw_line(pixmap, local_gc, x - delta, y, x + delta, y);
-    } else {
-	/* Draw in y axis */
-	delta = y_axis / 2 - x_axis / 2;
-	gdk_gc_set_line_attributes(local_gc, x_axis, 
-				   GDK_LINE_SOLID, 
-				   GDK_CAP_ROUND, 
-				   GDK_JOIN_MITER);
-	gdk_draw_line(pixmap, local_gc, x, y - delta, x, y + delta);
-    }
+	if (x_axis > y_axis) {
+		/* Draw in x axis */
+		width = y_axis;
+
+		points[0].x = -(x_axis >> 1) + (y_axis >> 1);
+		points[0].y = 0;
+		points[1].x =  (x_axis >> 1) - (y_axis >> 1);
+		points[1].y = 0;
+	} else {
+		/* Draw in y axis */
+		width = x_axis;
+
+		points[0].x = 0;
+		points[0].y = -(y_axis >> 1) + (x_axis >> 1);
+		points[1].x = 0;
+		points[1].y =  (y_axis >> 1) - (x_axis >> 1);
+	}
+
+	points[0] = rotate_point(points[0], angle_deg);
+	points[0].x += x;
+	points[0].y += y;
+	points[1] = rotate_point(points[1], angle_deg);
+	points[1].x += x;
+	points[1].y += y;
+
+	gdk_gc_set_line_attributes(local_gc, width,
+			GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_MITER);
+	gdk_draw_line(pixmap, local_gc,
+			points[0].x, points[0].y,
+			points[1].x, points[1].y);
 
     gdk_gc_unref(local_gc);
 
@@ -699,10 +712,8 @@ gerbv_gdk_draw_arc(GdkPixmap *pixmap, GdkGC *gc,
     gint real_x = x - width / 2;
     gint real_y = y - height / 2;
 
-    gdk_draw_arc(pixmap, gc, FALSE, real_x, real_y, width, height, 
-		 (gint)(angle1 * 64.0), (gint)(((angle2 - angle1) * 64.0)));
-    
-    return;
+    gdk_draw_arc(pixmap, gc, FALSE, real_x, real_y, width, height,
+		round(64*fmod(angle1, 360)), round(64*(angle2 - angle1)));
 } /* gerbv_gdk_draw_arc */
 
 void
@@ -712,8 +723,7 @@ draw_gdk_render_polygon_object (gerbv_net_t *oldNet, gerbv_image_t *image, doubl
 	gerbv_net_t *currentNet;
 	gint x2,y2,cp_x=0,cp_y=0,cir_width=0;
 	GdkPoint *points = NULL;
-	int pointArraySize=0;
-	int curr_point_idx = 0;
+	unsigned int pointArraySize, curr_point_idx;
 	int steps,i;
 	gdouble angleDiff, tempX, tempY;
 
@@ -736,6 +746,9 @@ draw_gdk_render_polygon_object (gerbv_net_t *oldNet, gerbv_image_t *image, doubl
 			tempX = currentNet->cirseg->width;
 			tempY = currentNet->cirseg->height;
 			cairo_matrix_transform_point (scaleMatrix, &tempX, &tempY);
+
+			/* Variables can be negative after transformation */
+			tempX = fabs(tempX);
 			cir_width = (int)round(tempX);
 
 			tempX = currentNet->cirseg->cp_x + sr_x;
@@ -746,13 +759,14 @@ draw_gdk_render_polygon_object (gerbv_net_t *oldNet, gerbv_image_t *image, doubl
 		}
 
 		switch (currentNet->interpolation) {
-			case GERBV_INTERPOLATION_x10 :
+			case GERBV_INTERPOLATION_LINEARx1 :
+			case GERBV_INTERPOLATION_LINEARx10 :
 			case GERBV_INTERPOLATION_LINEARx01 :
 			case GERBV_INTERPOLATION_LINEARx001 :
-			case GERBV_INTERPOLATION_LINEARx1 :
 				if (pointArraySize < (curr_point_idx + 1)) {
-					points = (GdkPoint *)g_realloc(points,sizeof(GdkPoint) *  (curr_point_idx + 1));
-					pointArraySize = (curr_point_idx + 1);
+					pointArraySize = curr_point_idx + 1;
+					points = (GdkPoint *)g_realloc(points,
+							pointArraySize*sizeof(GdkPoint));
 				}
 				points[curr_point_idx].x = x2;
 				points[curr_point_idx].y = y2;
@@ -765,14 +779,15 @@ draw_gdk_render_polygon_object (gerbv_net_t *oldNet, gerbv_image_t *image, doubl
 				angleDiff = currentNet->cirseg->angle2 - currentNet->cirseg->angle1;
 				steps = (int) abs(angleDiff);
 				if (pointArraySize < (curr_point_idx + steps)) {
-					points = (GdkPoint *)g_realloc(points,sizeof(GdkPoint) *  (curr_point_idx + steps));
-					pointArraySize = (curr_point_idx + steps);
+					pointArraySize = curr_point_idx + steps;
+					points = (GdkPoint *)g_realloc(points,
+							pointArraySize*sizeof(GdkPoint));
 				}
 				for (i=0; i<steps; i++){
-					points[curr_point_idx].x = cp_x + cir_width / 2.0 * cos ((currentNet->cirseg->angle1 +
-									      (angleDiff * i) / steps)*M_PI/180);
-					points[curr_point_idx].y = cp_y - cir_width / 2.0 * sin ((currentNet->cirseg->angle1 +
-									      (angleDiff * i) / steps)*M_PI/180);
+					points[curr_point_idx].x = cp_x + cir_width / 2.0 * cos (DEG2RAD(currentNet->cirseg->angle1 +
+									      (angleDiff * i) / steps));
+					points[curr_point_idx].y = cp_y - cir_width / 2.0 * sin (DEG2RAD(currentNet->cirseg->angle1 +
+									      (angleDiff * i) / steps));
 					curr_point_idx++;
 				}
 				break;
@@ -787,6 +802,9 @@ draw_gdk_render_polygon_object (gerbv_net_t *oldNet, gerbv_image_t *image, doubl
 				points = NULL;
 				return;
 			default:
+				GERB_COMPILE_WARNING(
+					_("Skipped interpolation type %d"),
+					currentNet->interpolation);
 				break;
 		}
 	}
@@ -822,21 +840,31 @@ draw_gdk_apply_netstate_transformation  (cairo_matrix_t *fullMatrix, cairo_matri
 	if (state->axisSelect == GERBV_AXIS_SELECT_SWAPAB) {
 		/* we do this by rotating 270 (counterclockwise, then mirroring
 		   the Y axis */
-		cairo_matrix_rotate (fullMatrix, 3 * M_PI / 2);
+		cairo_matrix_rotate (fullMatrix, M_PI + M_PI_2);
 		cairo_matrix_scale (fullMatrix, 1, -1);
 	}
 }
 
+static void
+draw_gdk_cross (GdkPixmap *pixmap, GdkGC *gc, gint xc, gint yc, gint r)
+{
+	gdk_gc_set_line_attributes (gc, 1, GDK_LINE_SOLID,
+			GDK_CAP_BUTT, GDK_JOIN_MITER);
+	gdk_draw_line (pixmap, gc, xc - r, yc, xc + r, yc);
+	gdk_draw_line (pixmap, gc, xc, yc - r, xc, yc + r);
+}
+
 /*
  * Convert a gerber image to a GDK clip mask to be used when creating pixmap
  */
 int
 draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image, 
 	     double scale, double trans_x, double trans_y,
-	     gchar drawMode,
+	     enum draw_mode drawMode,
 	     gerbv_selection_info_t *selectionInfo, gerbv_render_info_t *renderInfo,
 	     gerbv_user_transformation_t transform)
 {
+	const int hole_cross_inc_px = 8;
 	GdkGC *gc = gdk_gc_new(*pixmap);
 	GdkGC *pgc = gdk_gc_new(*pixmap);
 	GdkGCValues gc_values;
@@ -850,9 +878,16 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 	int cp_x = 0, cp_y = 0;
 	GdkColor transparent, opaque;
 	gerbv_polarity_t polarity;
-	gdouble tempX,tempY;
+	gdouble tempX, tempY, r;
 	gdouble minX=0,minY=0,maxX=0,maxY=0;
 
+	if (image == NULL || image->netlist == NULL) {
+		gdk_gc_unref(gc);
+		gdk_gc_unref(pgc);
+
+		return 0;
+	}
+
 	if (transform.inverted) {
 		if (image->info->polarity == GERBV_POLARITY_POSITIVE)
 			polarity = GERBV_POLARITY_NEGATIVE;
@@ -867,12 +902,12 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 	gboolean useOptimizations = TRUE;
 	// if the user is using any transformations for this layer, then don't bother using rendering
 	//   optimizations
-	if ((fabs(transform.translateX) > 0.00001) ||
-			(fabs(transform.translateY) > 0.00001) ||
-			(fabs(transform.scaleX - 1) > 0.00001) ||
-			(fabs(transform.scaleY - 1) > 0.00001) ||
-			(fabs(transform.rotation) > 0.00001) ||
-			transform.mirrorAroundX || transform.mirrorAroundY)
+	if (fabs(transform.translateX) > GERBV_PRECISION_LINEAR_INCH
+	||  fabs(transform.translateY) > GERBV_PRECISION_LINEAR_INCH
+	||  fabs(transform.scaleX - 1) > GERBV_PRECISION_LINEAR_INCH 
+	||  fabs(transform.scaleY - 1) > GERBV_PRECISION_LINEAR_INCH
+	||  fabs(transform.rotation) > GERBV_PRECISION_ANGLE_RAD
+	||  transform.mirrorAroundX || transform.mirrorAroundY)
 		useOptimizations = FALSE;
 
 	// calculate the transformation matrix for the user_transformation options
@@ -912,16 +947,6 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 					renderInfo->scaleFactorY);
 	}
 
-	if (image == NULL || image->netlist == NULL) {
-		/*
-		 * Destroy GCs before exiting
-		 */
-		gdk_gc_unref(gc);
-		gdk_gc_unref(pgc);
-
-		return 0;
-	}
-
 	/* Set up the two "colors" we have */
 	opaque.pixel = 0; /* opaque will not let color through */
 	transparent.pixel = 1; /* transparent will let color through */ 
@@ -970,12 +995,15 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 		if (drawMode == DRAW_SELECTIONS) {
 			int i;
 			gboolean foundNet = FALSE;
+			gerbv_selection_item_t sItem;
 			
 			for (i=0; i<selectionInfo->selectedNodeArray->len; i++){
-				gerbv_selection_item_t sItem = g_array_index (selectionInfo->selectedNodeArray,
-					gerbv_selection_item_t, i);
-				if (sItem.net == net)
+				sItem = g_array_index (selectionInfo->selectedNodeArray,
+						gerbv_selection_item_t, i);
+				if (sItem.net == net) {
 					foundNet = TRUE;
+					break;
+				}
 			}
 			if (!foundNet)
 				continue;
@@ -1000,7 +1028,11 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 		    tempX = net->cirseg->width;
 		    tempY = net->cirseg->height;
 		    cairo_matrix_transform_point (&scaleMatrix, &tempX, &tempY);
-		    cir_width = (int)round(tempX);
+
+		    /* Variables can be negative after transformation */
+		    tempX = fabs(tempX);
+		    tempY = fabs(tempY);
+		    cir_width =  (int)round(tempX);
 		    cir_height = (int)round(tempY);
 		    
 		    tempX = net->cirseg->cp_x;
@@ -1044,7 +1076,7 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 		if (image->aperture[net->aperture] == NULL) {
 		  /* Commenting this out since it gets emitted every time you click on the screen 
 		     if (net->aperture_state != GERBV_APERTURE_STATE_OFF)
-		     GERB_MESSAGE("Aperture D%d is not defined\n", net->aperture);
+		     GERB_MESSAGE("Aperture D%d is not defined", net->aperture);
 		  */
 		    continue;
 		}
@@ -1090,30 +1122,26 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 		if (ylong2 > G_MAXINT) y2 = G_MAXINT;
 		else if (ylong2 < G_MININT) y2 = G_MININT;
 		else y2 = (int)ylong2;
-			
+
 		switch (net->aperture_state) {
 		case GERBV_APERTURE_STATE_ON :
 		    tempX = image->aperture[net->aperture]->parameter[0];
 		    cairo_matrix_transform_point (&scaleMatrix, &tempX, &tempY);
+
+		    /* Variables can be negative after transformation */
+		    tempX = fabs(tempX);
 		    p1 = (int)round(tempX);
 
-		   // p1 = (int)round(image->aperture[net->aperture]->parameter[0] * scale);
-		    if (image->aperture[net->aperture]->type == GERBV_APTYPE_RECTANGLE)
-			gdk_gc_set_line_attributes(gc, p1, 
-						   GDK_LINE_SOLID, 
-						   GDK_CAP_PROJECTING, 
-						   GDK_JOIN_MITER);
-		    else
-			gdk_gc_set_line_attributes(gc, p1, 
-						   GDK_LINE_SOLID, 
-						   GDK_CAP_ROUND, 
-						   GDK_JOIN_MITER);
+		    gdk_gc_set_line_attributes(gc, p1, GDK_LINE_SOLID,
+				    (image->aperture[net->aperture]->type == GERBV_APTYPE_RECTANGLE)?
+						GDK_CAP_PROJECTING: GDK_CAP_ROUND,
+				    GDK_JOIN_MITER);
 		    
 		    switch (net->interpolation) {
-		    case GERBV_INTERPOLATION_x10 :
+		    case GERBV_INTERPOLATION_LINEARx10 :
 		    case GERBV_INTERPOLATION_LINEARx01 :
 		    case GERBV_INTERPOLATION_LINEARx001 :
-			GERB_MESSAGE(_("Linear != x1\n"));
+			GERB_MESSAGE(_("Linear != x1"));
 			gdk_gc_set_line_attributes(gc, p1, 
 						   GDK_LINE_ON_OFF_DASH, 
 						   GDK_CAP_ROUND, 
@@ -1125,35 +1153,52 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 						   GDK_JOIN_MITER);
 			break;
 		    case GERBV_INTERPOLATION_LINEARx1 :
-			if (image->aperture[net->aperture]->type != GERBV_APTYPE_RECTANGLE)
-			    gdk_draw_line(*pixmap, gc, x1, y1, x2, y2);
-			else {
-			    gint dx, dy;
-			    GdkPoint poly[6];
-			    
-			    tempX = image->aperture[net->aperture]->parameter[0]/2;
-			    tempY = image->aperture[net->aperture]->parameter[1]/2;
-			    cairo_matrix_transform_point (&scaleMatrix, &tempX, &tempY);
-			    dx = (int)round(tempX);
-			    dy = (int)round(tempY);
+			if (image->aperture[net->aperture]->type != GERBV_APTYPE_RECTANGLE) {
+				gdk_draw_line(*pixmap, gc, x1, y1, x2, y2);
 
-			    if(x1 > x2) dx = -dx;
-			    if(y1 > y2) dy = -dy;
-			    poly[0].x = x1 - dx; poly[0].y = y1 - dy;
-			    poly[1].x = x1 - dx; poly[1].y = y1 + dy;
-			    poly[2].x = x2 - dx; poly[2].y = y2 + dy;
-			    poly[3].x = x2 + dx; poly[3].y = y2 + dy;
-			    poly[4].x = x2 + dx; poly[4].y = y2 - dy;
-			    poly[5].x = x1 + dx; poly[5].y = y1 - dy;
-			    gdk_draw_polygon(*pixmap, gc, 1, poly, 6);
+				if (renderInfo->show_cross_on_drill_holes
+				&& image->layertype == GERBV_LAYERTYPE_DRILL) {
+					/* Draw crosses on drill slot start and end */
+					r = p1/2.0 + hole_cross_inc_px;
+					draw_gdk_cross(*pixmap, gc, x1, y1, r);
+					draw_gdk_cross(*pixmap, gc, x2, y2, r);
+				}
+			    break;
 			}
-				break;
+
+			gint dx, dy;
+			GdkPoint poly[6];
+
+			tempX = image->aperture[net->aperture]->parameter[0]/2;
+			tempY = image->aperture[net->aperture]->parameter[1]/2;
+			cairo_matrix_transform_point (&scaleMatrix, &tempX, &tempY);
+			dx = (int)round(tempX);
+			dy = (int)round(tempY);
+
+			if(x1 > x2) dx = -dx;
+			if(y1 > y2) dy = -dy;
+			poly[0].x = x1 - dx; poly[0].y = y1 - dy;
+			poly[1].x = x1 - dx; poly[1].y = y1 + dy;
+			poly[2].x = x2 - dx; poly[2].y = y2 + dy;
+			poly[3].x = x2 + dx; poly[3].y = y2 + dy;
+			poly[4].x = x2 + dx; poly[4].y = y2 - dy;
+			poly[5].x = x1 + dx; poly[5].y = y1 - dy;
+			gdk_draw_polygon(*pixmap, gc, 1, poly, 6);
+			break;
+
 		    case GERBV_INTERPOLATION_CW_CIRCULAR :
 		    case GERBV_INTERPOLATION_CCW_CIRCULAR :
-			gerbv_gdk_draw_arc(*pixmap, gc, cp_x, cp_y, cir_width, cir_height, 
-					   net->cirseg->angle1, net->cirseg->angle2);
+			gerbv_gdk_draw_arc(*pixmap, gc, cp_x, cp_y,
+				cir_width, cir_height,
+				net->cirseg->angle1 +
+					RAD2DEG(transform.rotation),
+				net->cirseg->angle2 +
+					RAD2DEG(transform.rotation));
 			break;
 		    default :
+			GERB_COMPILE_WARNING(
+				_("Skipped interpolation type %d"),
+				net->interpolation);
 			break;
 		    }
 		    break;
@@ -1163,14 +1208,23 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 		    tempX = image->aperture[net->aperture]->parameter[0];
 		    tempY = image->aperture[net->aperture]->parameter[1];
 		    cairo_matrix_transform_point (&scaleMatrix, &tempX, &tempY);
+
+		    /* Variables can be negative after transformation */
+		    tempX = fabs(tempX);
+		    tempY = fabs(tempY);
 		    p1 = (int)round(tempX);
 		    p2 = (int)round(tempY);
-		    tempX = image->aperture[net->aperture]->parameter[2];
-		    cairo_matrix_transform_point (&scaleMatrix, &tempX, &tempY);
 		    
 		    switch (image->aperture[net->aperture]->type) {
 		    case GERBV_APTYPE_CIRCLE :
 			gerbv_gdk_draw_circle(*pixmap, gc, TRUE, x2, y2, p1);
+
+			if (renderInfo->show_cross_on_drill_holes
+			&& image->layertype == GERBV_LAYERTYPE_DRILL) {
+				r = p1/2.0 + hole_cross_inc_px;
+				draw_gdk_cross(*pixmap, gc, x2, y2, r);
+			}
+
 			/*
 			 * If circle has an inner diameter we must remove
 			 * that part of the circle to make a hole in it.
@@ -1192,26 +1246,34 @@ draw_gdk_image_to_pixmap(GdkPixmap **pixmap, gerbv_image_t *image,
 
 			break;
 		    case GERBV_APTYPE_RECTANGLE:
-			gerbv_gdk_draw_rectangle(*pixmap, gc, TRUE, x2, y2, p1, p2);
+			gerbv_gdk_draw_rectangle(*pixmap, gc, TRUE,
+				x2, y2, p1, p2, RAD2DEG(transform.rotation +
+					image->info->imageRotation));
 			break;
 		    case GERBV_APTYPE_OVAL :
-			gerbv_gdk_draw_oval(*pixmap, gc, TRUE, x2, y2, p1, p2);
+			gerbv_gdk_draw_oval(*pixmap, gc, TRUE,
+				x2, y2, p1, p2, RAD2DEG(transform.rotation +
+					image->info->imageRotation));
 			break;
 		    case GERBV_APTYPE_POLYGON :
+			/* TODO: gdk_draw_polygon() */
 			gerbv_gdk_draw_circle(*pixmap, gc, TRUE, x2, y2, p1);
 			break;
 		    case GERBV_APTYPE_MACRO :
+			/* TODO: check line22 and others */
 			gerbv_gdk_draw_amacro(*pixmap, gc, 
 					      image->aperture[net->aperture]->simplified,
 					      scale, x2, y2);
 			break;
 		    default :
-			GERB_MESSAGE(_("Unknown aperture type\n"));
+			GERB_MESSAGE(_("Unknown aperture type %d"),
+					image->aperture[net->aperture]->type);
 			return 0;
 		    }
 		    break;
 		default :
-		    GERB_MESSAGE(_("Unknown aperture state\n"));
+		    GERB_MESSAGE(_("Unknown aperture state %d"),
+				net->aperture_state);
 		    return 0;
 		}
 		}
