@@ -27,21 +27,15 @@
 */
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <glib.h>
-#include <math.h>
-#include <locale.h>
-
-#include <glib/gstdio.h>
 #include "gerbv.h"
-#include "common.h"
-#include "draw-gdk.h"
 
-/* DEBUG printing.  #define DEBUG 1 in config.h to use this fcn. */
+#include <math.h>
+#include <glib/gstdio.h>
+
+#include "common.h"
+
 #define dprintf if(DEBUG) printf
+
 #define round(x) floor(x+0.5)
 
 void
@@ -61,7 +55,8 @@ export_rs274x_write_macro (FILE *fd, gerbv_aperture_t *currentAperture,
 			int pointCounter;
 			int numberOfPoints = (int) ls->parameter[OUTLINE_NUMBER_OF_POINTS];
 			
-			fprintf(fd, "4,%d,%d,\n",(int) ls->parameter[OUTLINE_EXPOSURE],
+			/* for Flatcam no new line after this 3 digits */
+			fprintf(fd, "4,%d,%d,",(int) ls->parameter[OUTLINE_EXPOSURE],
 				numberOfPoints);
 			/* add 1 point for the starting point here */
 			for (pointCounter=0; pointCounter <= numberOfPoints; pointCounter++) {
@@ -195,8 +190,10 @@ export_rs274x_write_state_change (gerbv_netstate_t *oldState, gerbv_netstate_t *
 }
 
 gboolean
-gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
-		gerbv_user_transformation_t *transform) {
+gerbv_export_rs274x_file_from_image (const gchar *filename, gerbv_image_t *inputImage,
+		gerbv_user_transformation_t *transform)
+{
+	const double decimal_coeff = 1e6;
 	FILE *fd;
 	gerbv_netstate_t *oldState;
 	gerbv_layer_t *oldLayer;
@@ -208,13 +205,14 @@ gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
 
 	if (transform != NULL) {
 		thisTransform = transform;
-	}
-	else {
-		gerbv_user_transformation_t identityTransform = {0,0,1,1,0,FALSE,FALSE,FALSE};
+	} else {
+		static gerbv_user_transformation_t identityTransform =
+						{0,0,1,1,0,FALSE,FALSE,FALSE};
 		thisTransform = &identityTransform;
 	}
 	if ((fd = g_fopen(filename, "w")) == NULL) {
-		GERB_MESSAGE(_("Can't open file for writing: %s\n"), filename);
+		GERB_COMPILE_ERROR(_("Can't open file for writing: %s"),
+				filename);
 		return FALSE;
 	}
 	
@@ -228,7 +226,7 @@ gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
 	fprintf(fd, "G04 http://gerbv.geda-project.org/ *\n");
 	fprintf(fd, "G04 --End of header info--*\n");
 	fprintf(fd, "%%MOIN*%%\n");
-	fprintf(fd, "%%FSLAX34Y34*%%\n");
+	fprintf(fd, "%%FSLAX36Y36*%%\n");
 	
 	/* check the image info struct for any non-default settings */
 	/* image offset */
@@ -245,11 +243,16 @@ gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
 	/* plotter film */
 	if (image->info->plotterFilm)
 		fprintf(fd, "%%PF%s*%%\n",image->info->plotterFilm);
+
 	/* image rotation */
-	if ((image->info->imageRotation != 0.0)||(thisTransform->rotation != 0.0))
-		fprintf(fd, "%%IR%d*%%\n",(int) ((image->info->imageRotation + thisTransform->rotation)*180/M_PI)%360);
-	if ((image->info->imageJustifyTypeA != GERBV_JUSTIFY_NOJUSTIFY) ||
-		(image->info->imageJustifyTypeB != GERBV_JUSTIFY_NOJUSTIFY)) {
+	if ((image->info->imageRotation != 0.0)
+	||  (thisTransform->rotation != 0.0))
+		fprintf(fd, "%%IR%d*%%\n",
+			(int)(RAD2DEG(image->info->imageRotation +
+					thisTransform->rotation))%360);
+
+	if ((image->info->imageJustifyTypeA != GERBV_JUSTIFY_NOJUSTIFY)
+	||  (image->info->imageJustifyTypeB != GERBV_JUSTIFY_NOJUSTIFY)) {
 		fprintf(fd, "%%IJA");
 		if (image->info->imageJustifyTypeA == GERBV_JUSTIFY_CENTERJUSTIFY)
 			fprintf(fd, "C");
@@ -264,8 +267,8 @@ gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
 
 	}
 	/* handle scale user orientation transforms */
-	if ((fabs(thisTransform->scaleX - 1) > 0.00001) ||
-	(fabs(thisTransform->scaleY - 1) > 0.00001)) {
+	if (fabs(thisTransform->scaleX - 1) > GERBV_PRECISION_LINEAR_INCH
+	||  fabs(thisTransform->scaleY - 1) > GERBV_PRECISION_LINEAR_INCH) {
 		fprintf(fd, "%%SFA%.4fB%.4f*%%\n",thisTransform->scaleX,thisTransform->scaleY);
 	}
 	/* handle mirror image user orientation transform */
@@ -308,19 +311,19 @@ gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
 		
 		long xVal,yVal,endX,endY,centerX,centerY;
 		switch (currentNet->interpolation) {
-			case GERBV_INTERPOLATION_x10 :
+			case GERBV_INTERPOLATION_LINEARx1 :
+			case GERBV_INTERPOLATION_LINEARx10 :
 			case GERBV_INTERPOLATION_LINEARx01 :
 			case GERBV_INTERPOLATION_LINEARx001 :
-			case GERBV_INTERPOLATION_LINEARx1 :
 				/* see if we need to write an "aperture off" line to get
 				   the pen to the right start point */
 				if ((!insidePolygon) && (currentNet->aperture_state == GERBV_APERTURE_STATE_ON)) {
-					xVal = (long) round(currentNet->start_x * 10000.0);
-					yVal = (long) round(currentNet->start_y * 10000.0);
+					xVal = (long) round(currentNet->start_x * decimal_coeff);
+					yVal = (long) round(currentNet->start_y * decimal_coeff);
 					fprintf(fd, "G01X%07ldY%07ldD02*\n",xVal,yVal);
 				}
-				xVal = (long) round(currentNet->stop_x * 10000.0);
-				yVal = (long) round(currentNet->stop_y * 10000.0);
+				xVal = (long) round(currentNet->stop_x * decimal_coeff);
+				yVal = (long) round(currentNet->stop_y * decimal_coeff);
 				fprintf(fd, "G01X%07ldY%07ld",xVal,yVal);
 				/* and finally, write the esposure value */
 				if (currentNet->aperture_state == GERBV_APERTURE_STATE_OFF)
@@ -335,23 +338,24 @@ gerbv_export_rs274x_file_from_image (gchar *filename, gerbv_image_t *inputImage,
 				/* see if we need to write an "aperture off" line to get
 				   the pen to the right start point */
 				if ((!insidePolygon) && (currentNet->aperture_state == GERBV_APERTURE_STATE_ON)) {
-					xVal = (long) round(currentNet->start_x * 10000.0);
-					yVal = (long) round(currentNet->start_y * 10000.0);
+					xVal = (long) round(currentNet->start_x * decimal_coeff);
+					yVal = (long) round(currentNet->start_y * decimal_coeff);
 					fprintf(fd, "G01X%07ldY%07ldD02*\n",xVal,yVal);
 				}
-				centerX= (long) round((currentNet->cirseg->cp_x - currentNet->start_x) * 10000.0);
-				centerY= (long) round((currentNet->cirseg->cp_y - currentNet->start_y) * 10000.0);
-				endX = (long) round(currentNet->stop_x * 10000.0);
-				endY = (long) round(currentNet->stop_y * 10000.0);
+				centerX= (long) round((currentNet->cirseg->cp_x - currentNet->start_x) * decimal_coeff);
+				centerY= (long) round((currentNet->cirseg->cp_y - currentNet->start_y) * decimal_coeff);
+				endX = (long) round(currentNet->stop_x * decimal_coeff);
+				endY = (long) round(currentNet->stop_y * decimal_coeff);
 				
 				/* always use multi-quadrant, since it's much easier to export */
 				/*  and most all software should support it */
 				fprintf(fd, "G75*\n");
-				/* figure out clockwise or c-clockwise */
-				if (currentNet->cirseg->angle2 > currentNet->cirseg->angle1)
-					fprintf(fd, "G03");
+
+				if (currentNet->interpolation == GERBV_INTERPOLATION_CW_CIRCULAR)
+					fprintf(fd, "G02");	/* Clockwise */
 				else
-					fprintf(fd, "G02");
+					fprintf(fd, "G03");	/* Counter clockwise */
+
 				/* don't write the I and J values if the exposure is off */
 				if (currentNet->aperture_state == GERBV_APERTURE_STATE_ON)
 					fprintf(fd, "X%07ldY%07ldI%07ldJ%07ld",endX,endY,centerX,centerY);
