@@ -43,6 +43,7 @@
 #include <sys/mman.h>
 #endif
 #include <errno.h>
+#include <glib/gstdio.h>
 
 #include "common.h"
 #include "gerbv.h"
@@ -59,7 +60,7 @@ gerb_fopen(char const * filename)
     
     dprintf("---> Entering gerb_fopen, filename = %s\n", filename);
 #ifdef HAVE_SYS_MMAN_H
-    fd = (gerb_file_t *)g_malloc(sizeof(gerb_file_t));
+    fd = g_new(gerb_file_t, 1);
     if (fd == NULL) {
 	return NULL;
     }
@@ -107,18 +108,17 @@ gerb_fopen(char const * filename)
     }
 #else
     /* all systems without mmap, not only MINGW32 */
-    fd = (gerb_file_t *)g_malloc(sizeof(gerb_file_t));
+    fd = g_new(gerb_file_t);
     if (fd == NULL) {
 	return NULL;
     }
 
-    if (stat(filename, &statinfo) < 0) {
-	fclose(fd->fd);
+    /* fopen() can't open files with non ASCII filenames on windows */
+    fd->fd = g_fopen(filename, "rb");
+    if (fd->fd == NULL) {
 	g_free(fd);
-        return NULL;
+	return NULL;
     }
-    
-    fd->fd = fopen(filename, "rb");
     fd->ptr = 0;
     fd->fileno = fileno(fd->fd);
     if (fstat(fd->fileno, &statinfo) < 0) {
