@@ -26,16 +26,12 @@
     \ingroup libgerbv
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "gerbv.h"
 
 #include <stdlib.h>
 #include <string.h>
-#include <glib.h>
 #include <math.h>
 
-#include "gerbv.h"
 #include "common.h"
 #include "gerb_image.h"
 #include "gerber.h"
@@ -52,26 +48,23 @@ gerbv_create_image(gerbv_image_t *image, const gchar *type)
     gerbv_destroy_image(image);
     
     /* Malloc space for image */
-    if ((image = (gerbv_image_t *)g_malloc(sizeof(gerbv_image_t))) == NULL) {
+    if (NULL == (image = g_new0(gerbv_image_t, 1))) {
 	return NULL;
     }
-    memset((void *)image, 0, sizeof(gerbv_image_t));
-    
+
     /* Malloc space for image->netlist */
-    if ((image->netlist = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t))) == NULL) {
+    if (NULL == (image->netlist = g_new0(gerbv_net_t, 1))) {
 	g_free(image);
 	return NULL;
     }
-    memset((void *)image->netlist, 0, sizeof(gerbv_net_t));
-    
+
     /* Malloc space for image->info */
-    if ((image->info = (gerbv_image_info_t *)g_malloc(sizeof(gerbv_image_info_t))) == NULL) {
+    if (NULL == (image->info = g_new0(gerbv_image_info_t, 1))) {
 	g_free(image->netlist);
 	g_free(image);
 	return NULL;
     }
-    memset((void *)image->info, 0, sizeof(gerbv_image_info_t));
-    
+
     /* Set aside position for stats struct */
     image->gerbv_stats = NULL;
     image->drill_stats = NULL;
@@ -239,40 +232,6 @@ gerbv_image_verify(gerbv_image_t const* image)
     return error;
 } /* gerb_image_verify */
 
-
-static void
-gerbv_image_interpolation(gerbv_interpolation_t interpolation)
-{
-    switch (interpolation) {
-    case GERBV_INTERPOLATION_LINEARx1:
-	printf(_("linearX1"));
-	break;
-    case GERBV_INTERPOLATION_x10:
-	printf(_("linearX10"));
-	break;
-    case GERBV_INTERPOLATION_LINEARx01:
-	printf(_("linearX01"));
-	break;
-    case GERBV_INTERPOLATION_LINEARx001:
-	printf(_("linearX001"));
-	break;
-    case GERBV_INTERPOLATION_CW_CIRCULAR:
-	printf(_("CW circular"));
-	break;
-    case GERBV_INTERPOLATION_CCW_CIRCULAR:
-	printf(_("CCW circular"));
-	break;
-    case  GERBV_INTERPOLATION_PAREA_START:
-	printf(_("polygon area start"));
-	break;
-    case  GERBV_INTERPOLATION_PAREA_END:
-	printf(_("polygon area end"));
-	break;
-    default:
-	printf(_("unknown"));
-    }
-} /* gerb_image_interpolation */
-
 static void
 gerbv_image_aperture_state(gerbv_aperture_state_t state)
 {
@@ -336,7 +295,7 @@ gerbv_image_dump(gerbv_image_t const* image)
     while (net){
 	printf(_("(%f,%f)->(%f,%f) with %d ("), net->start_x, net->start_y,
 	       net->stop_x, net->stop_y, net->aperture);
-	gerbv_image_interpolation(net->interpolation);
+	printf(_(gerbv_interpolation_name(net->interpolation)));
 	gerbv_image_aperture_state(net->aperture_state);
 	printf(")\n");
 	net = net->next;
@@ -382,21 +341,23 @@ gerbv_image_duplicate_layer (gerbv_layer_t *oldLayer) {
     return newLayer;
 }
 
-gerbv_netstate_t *
-gerbv_image_duplicate_state (gerbv_netstate_t *oldState) {
-    gerbv_netstate_t *newState = g_new (gerbv_netstate_t,1);
-    
-    *newState = *oldState;
-    return newState;
+static gerbv_netstate_t *
+gerbv_image_duplicate_state (gerbv_netstate_t *oldState)
+{
+	gerbv_netstate_t *newState = g_new (gerbv_netstate_t, 1);
+
+	*newState = *oldState;
+	return newState;
 }
 
-gerbv_aperture_t *
-gerbv_image_duplicate_aperture (gerbv_aperture_t *oldAperture){
+static gerbv_aperture_t *
+gerbv_image_duplicate_aperture (gerbv_aperture_t *oldAperture)
+{
     gerbv_aperture_t *newAperture = g_new0 (gerbv_aperture_t,1);
     gerbv_simplified_amacro_t *simplifiedMacro, *tempSimplified;
-       
+
     *newAperture = *oldAperture;
-	  
+
     /* delete the amacro section, since we really don't need it anymore
     now that we have the simplified section */
     newAperture->amacro = NULL;
@@ -416,76 +377,493 @@ gerbv_image_duplicate_aperture (gerbv_aperture_t *oldAperture){
     return newAperture;
 }
 
-void
-gerbv_image_copy_all_nets (gerbv_image_t *sourceImage, gerbv_image_t *newImage, gerbv_layer_t *lastLayer,
-		gerbv_netstate_t *lastState, gerbv_net_t *lastNet, gerbv_user_transformation_t *transform,
-		GArray *translationTable){
-    gerbv_netstate_t *oldState,*newSavedState;
-    gerbv_layer_t *oldLayer,*newSavedLayer;
-    gerbv_net_t *currentNet,*newNet,*newSavedNet;
-    int i;
-    
-    oldLayer = sourceImage->layers;
-    oldState = sourceImage->states;
-    
-    newSavedLayer = lastLayer;
-    newSavedState = lastState;
-    newSavedNet = lastNet;
-    
-    for (currentNet = sourceImage->netlist; currentNet; currentNet = currentNet->next){
-      /* check for any new layers and duplicate them if needed */
-	if (currentNet->layer != oldLayer) {
-	  newSavedLayer->next = gerbv_image_duplicate_layer (currentNet->layer);
-	  newSavedLayer = newSavedLayer->next;
+static void
+gerbv_image_copy_all_nets (gerbv_image_t *sourceImage,
+		gerbv_image_t *destImage, gerbv_layer_t *lastLayer,
+		gerbv_netstate_t *lastState, gerbv_net_t *lastNet,
+		gerbv_user_transformation_t *trans,
+		GArray *translationTable)
+{
+	/* NOTE: destImage already contains apertures and data,
+	 * latest data is: lastLayer, lastState, lastNet. */
+
+	gerbv_net_t *currentNet, *newNet;
+	gerbv_aperture_type_t aper_type;
+	gerbv_aperture_t *aper;
+	gerbv_simplified_amacro_t *sam;
+	int *trans_apers = NULL; /* Transformed apertures */
+	int aper_last_id = 0;
+	guint	err_scale_circle = 0,
+		err_scale_line_macro = 0,
+		err_scale_poly_macro = 0,
+		err_scale_thermo_macro = 0,
+		err_scale_moire_macro = 0,
+		err_unknown_aperture = 0,
+		err_unknown_macro_aperture = 0,
+		err_rotate_oval = 0,
+		err_rotate_rect = 0;
+	guint i;
+
+	if (trans && (trans->mirrorAroundX || trans->mirrorAroundY)) {
+		if (sourceImage->layertype != GERBV_LAYERTYPE_DRILL) {
+			GERB_COMPILE_ERROR(_("Exporting mirrored file "
+						"is not supported!"));
+			return;
+		}
 	}
-	/* check for any new states and duplicate them if needed */
-	if (currentNet->state != oldState) {
-	  newSavedState->next = gerbv_image_duplicate_state (currentNet->state);
-	  newSavedState = newSavedState->next;
-      }
-      /* create and copy the actual net over */
-      newNet = g_new (gerbv_net_t,1);
-      *newNet = *currentNet;
-      
-      if (currentNet->cirseg) {
-      	newNet->cirseg = g_new (gerbv_cirseg_t,1);
-      	*(newNet->cirseg) = *(currentNet->cirseg);
-      }
-      
-      if (currentNet->label)
-      	newNet->label = g_string_new(currentNet->label->str);
-      
-      newNet->state = newSavedState;
-      newNet->layer = newSavedLayer;
-      /* check if we need to translate the aperture number */
-      if (translationTable) {
-        for (i=0; i<translationTable->len; i++){
-          gerb_translation_entry_t translationEntry=g_array_index (translationTable, gerb_translation_entry_t, i);
-          
-          if (translationEntry.oldAperture == newNet->aperture) {
-            newNet->aperture = translationEntry.newAperture;
-            break;
-          }
-        }
-      }
-      /* check if we are transforming the net (translating, scaling, etc) */
-      if (transform) {
-        newNet->start_x += transform->translateX;
-        newNet->start_y += transform->translateY;
-        newNet->stop_x += transform->translateX;
-        newNet->stop_y += transform->translateY;
-        if (newNet->cirseg) {
-          newNet->cirseg->cp_x += transform->translateX;
-          newNet->cirseg->cp_y += transform->translateY;
-        }
-      }
-      if (newSavedNet)
-      	newSavedNet->next = newNet;
-      else
-      	newImage->netlist = newNet;
-      newSavedNet = newNet;
-    }		
-		
+
+	if (trans && trans->inverted) {
+		GERB_COMPILE_ERROR(_("Exporting inverted file "
+					"is not supported!"));
+		return;
+	}
+
+	if (trans) {
+		/* Find last used aperture to add transformed apertures if
+		 * needed */
+		for (aper_last_id = APERTURE_MAX - 1; aper_last_id > 0;
+						aper_last_id--) {
+			if (destImage->aperture[aper_last_id] != NULL)
+				break;
+		}
+
+		trans_apers = g_new (int, aper_last_id + 1);
+		/* Initialize trans_apers array */
+		for (i = 0; i < aper_last_id + 1; i++)
+			trans_apers[i] = -1;
+	}
+
+	for (currentNet = sourceImage->netlist; currentNet != NULL;
+			currentNet = currentNet->next) {
+
+		/* Check for any new layers and duplicate them if needed */
+		if (currentNet->layer != lastLayer) {
+			lastLayer->next =
+				gerbv_image_duplicate_layer (currentNet->layer);
+			lastLayer = lastLayer->next;
+		}
+
+		/* Check for any new states and duplicate them if needed */
+		if (currentNet->state != lastState) {
+			lastState->next =
+				gerbv_image_duplicate_state (currentNet->state);
+			lastState = lastState->next;
+		}
+
+		/* Create and copy the actual net over */
+		newNet = g_new (gerbv_net_t, 1);
+		*newNet = *currentNet;
+
+		if (currentNet->cirseg) {
+			newNet->cirseg = g_new (gerbv_cirseg_t, 1);
+			*(newNet->cirseg) = *(currentNet->cirseg);
+		}
+
+		if (currentNet->label)
+			newNet->label = g_string_new (currentNet->label->str);
+		else
+			newNet->label = NULL;
+
+		newNet->state = lastState;
+		newNet->layer = lastLayer;
+
+		if (lastNet)
+			lastNet->next = newNet;
+		else
+			destImage->netlist = newNet;
+
+		lastNet = newNet;
+
+		/* Check if we need to translate the aperture number */
+		if (translationTable) {
+			for (i = 0; i < translationTable->len; i++) {
+				gerb_translation_entry_t translationEntry;
+
+				translationEntry =
+					g_array_index (translationTable,
+						gerb_translation_entry_t, i);
+
+				if (translationEntry.oldAperture ==
+						newNet->aperture) {
+					newNet->aperture =
+						translationEntry.newAperture;
+					break;
+				}
+			}
+		}
+
+		if (trans == NULL)
+			continue;
+
+		/* Transforming coords */
+		gerbv_transform_coord (&newNet->start_x,
+				&newNet->start_y, trans);
+		gerbv_transform_coord (&newNet->stop_x,
+				&newNet->stop_y, trans);
+
+		if (newNet->cirseg) {
+			/* Circular interpolation only exported by start, stop
+			 * end center coordinates. */
+			gerbv_transform_coord (&newNet->cirseg->cp_x,
+				&newNet->cirseg->cp_y, trans);
+		}
+
+		if (destImage->aperture[newNet->aperture] == NULL)
+			continue;
+
+		if (trans->scaleX == 1.0 && trans->scaleY == 1.0
+		&& fabs(trans->rotation) < GERBV_PRECISION_ANGLE_RAD)
+			continue;
+
+		/* Aperture is already transformed, use it */
+		if (trans_apers[newNet->aperture] != -1) {
+			newNet->aperture = trans_apers[newNet->aperture];
+			continue;
+		}
+
+		/* Transforming apertures */
+		aper_type = destImage->aperture[newNet->aperture]->type;
+		switch (aper_type) {
+		case GERBV_APTYPE_NONE:
+		case GERBV_APTYPE_POLYGON:
+			break;
+
+		case GERBV_APTYPE_CIRCLE:
+			if (trans->scaleX == trans->scaleY
+					&& trans->scaleX == 1.0) {
+				break;
+			}
+
+			if (trans->scaleX == trans->scaleY) {
+				aper = gerbv_image_duplicate_aperture (
+						destImage->aperture[
+							newNet->aperture]);
+				aper->parameter[0] *= trans->scaleX;
+
+				trans_apers[newNet->aperture] = ++aper_last_id;
+				destImage->aperture[aper_last_id] = aper;
+				newNet->aperture = aper_last_id;
+			} else {
+				err_scale_circle++;
+			}
+			break;
+
+		case GERBV_APTYPE_RECTANGLE:
+		case GERBV_APTYPE_OVAL:
+			if (trans->scaleX == 1.0 && trans->scaleY == 1.0
+			&&  fabs(fabs(trans->rotation) - M_PI)
+						< GERBV_PRECISION_ANGLE_RAD)
+				break;
+
+			aper = gerbv_image_duplicate_aperture (
+					destImage->aperture[newNet->aperture]);
+			aper->parameter[0] *= trans->scaleX;
+			aper->parameter[1] *= trans->scaleY;
+
+			if (fabs(fabs(trans->rotation) - M_PI_2)
+						< GERBV_PRECISION_ANGLE_RAD
+			||  fabs(fabs(trans->rotation) - (M_PI+M_PI_2))
+						< GERBV_PRECISION_ANGLE_RAD) {
+				double t = aper->parameter[0];
+				aper->parameter[0] = aper->parameter[1];
+				aper->parameter[1] = t;
+			} else {
+				if (aper_type == GERBV_APTYPE_RECTANGLE)
+					err_rotate_rect++;	/* TODO: make line21 macro */
+				else
+					err_rotate_oval++;
+
+				break;
+			}
+
+			trans_apers[newNet->aperture] = ++aper_last_id;
+			destImage->aperture[aper_last_id] = aper;
+			newNet->aperture = aper_last_id;
+
+			break;
+
+		case GERBV_APTYPE_MACRO:
+			aper = gerbv_image_duplicate_aperture (
+					destImage->aperture[newNet->aperture]);
+			sam = aper->simplified;
+
+			for (; sam != NULL; sam = sam->next) {
+				switch (sam->type) {
+				case GERBV_APTYPE_MACRO_CIRCLE:
+
+/* TODO: test circle macro center rotation */
+					sam->parameter[CIRCLE_CENTER_X] *=
+								trans->scaleX;
+					sam->parameter[CIRCLE_CENTER_Y] *=
+								trans->scaleY;
+					gerbv_rotate_coord(
+						sam->parameter +CIRCLE_CENTER_X,
+						sam->parameter +CIRCLE_CENTER_Y,
+						trans->rotation);
+
+					if (trans->scaleX != trans->scaleY) {
+						err_scale_circle++;
+						break;
+					}
+					sam->parameter[CIRCLE_DIAMETER] *=
+								trans->scaleX;
+					break;
+
+				case GERBV_APTYPE_MACRO_LINE20:
+					/* Vector line rectangle */
+					if (trans->scaleX == trans->scaleY) {
+						sam->parameter[LINE20_LINE_WIDTH] *=
+								trans->scaleX;
+					} else if (sam->parameter[LINE20_START_X] ==
+							sam->parameter[LINE20_END_X]) {
+						sam->parameter[LINE20_LINE_WIDTH] *=
+							trans->scaleX;	/* Vertical */
+					} else if (sam->parameter[LINE20_START_Y] ==
+							sam->parameter[LINE20_END_Y]) {
+						sam->parameter[LINE20_LINE_WIDTH] *=
+							trans->scaleY;	/* Horizontal */
+					} else {
+						/* TODO: make outline macro */
+						err_scale_line_macro++;
+						break;
+					}
+
+					sam->parameter[LINE20_START_X] *=
+							trans->scaleX;
+					sam->parameter[LINE20_START_Y] *=
+							trans->scaleY;
+					sam->parameter[LINE20_END_X] *=
+							trans->scaleX;
+					sam->parameter[LINE20_END_Y] *=
+							trans->scaleY;
+
+					/* LINE20_START_X, LINE20_START_Y,
+					 * LINE20_END_X, LINE20_END_Y are not
+					 * rotated, change only rotation angle */
+					sam->parameter[LINE20_ROTATION] +=
+						RAD2DEG(trans->rotation);
+					break;
+
+/* Compile time check if LINE21 and LINE22 parameters indexes are equal */
+#if (LINE21_WIDTH != LINE22_WIDTH) \
+ || (LINE21_HEIGHT != LINE22_HEIGHT) \
+ || (LINE21_ROTATION != LINE22_ROTATION) \
+ || (LINE21_CENTER_X != LINE22_LOWER_LEFT_X) \
+ || (LINE21_CENTER_Y != LINE22_LOWER_LEFT_Y)
+# error "LINE21 and LINE22 indexes are not equal"
+#endif
+
+				case GERBV_APTYPE_MACRO_LINE21:
+						/* Centered line rectangle */
+				case GERBV_APTYPE_MACRO_LINE22:
+						/* Lower left line rectangle */
+
+					/* Using LINE21 parameters array
+					 * indexes for LINE21 and LINE22, as
+					 * they are equal */
+					if (trans->scaleX == trans->scaleY) {
+						sam->parameter[LINE21_WIDTH] *=
+								trans->scaleX;
+						sam->parameter[LINE21_HEIGHT] *=
+								trans->scaleX;
+
+					} else if (fabs(sam->parameter[LINE21_ROTATION]) == 0
+					|| fabs(sam->parameter[LINE21_ROTATION]) == 180) {
+						sam->parameter[LINE21_WIDTH] *=
+								trans->scaleX;
+						sam->parameter[LINE21_HEIGHT] *=
+								trans->scaleY;
+
+					} else if (fabs(sam->parameter[LINE21_ROTATION]) == 90
+					|| fabs(sam->parameter[LINE21_ROTATION]) == 270) {
+						double t;
+						t = sam->parameter[LINE21_WIDTH];
+						sam->parameter[LINE21_WIDTH] =
+							trans->scaleY *
+							sam->parameter[
+								LINE21_HEIGHT];
+						sam->parameter[LINE21_HEIGHT] =
+							trans->scaleX * t;
+					} else {
+						/* TODO: make outline macro */
+						err_scale_line_macro++;
+						break;
+					}
+
+					sam->parameter[LINE21_CENTER_X] *=
+								trans->scaleX;
+					sam->parameter[LINE21_CENTER_Y] *=
+								trans->scaleY;
+
+					sam->parameter[LINE21_ROTATION] +=
+						RAD2DEG(trans->rotation);
+					gerbv_rotate_coord(
+						sam->parameter +LINE21_CENTER_X,
+						sam->parameter +LINE21_CENTER_Y,
+						trans->rotation);
+					break;
+
+				case GERBV_APTYPE_MACRO_OUTLINE:
+					for (i = 0; i < 1 + sam->parameter[
+							OUTLINE_NUMBER_OF_POINTS]; i++) {
+						sam->parameter[OUTLINE_X_IDX_OF_POINT(i)] *=
+								trans->scaleX;
+						sam->parameter[OUTLINE_Y_IDX_OF_POINT(i)] *=
+								trans->scaleY;
+					}
+
+					sam->parameter[OUTLINE_ROTATION_IDX(sam->parameter)] +=
+								RAD2DEG(trans->rotation);
+					break;
+#if 0
+{
+/* TODO */
+#include "main.h"
+gerbv_selection_item_t sItem = {sourceImage, currentNet};
+selection_add_item (&screen.selectionInfo, &sItem);
+}
+#endif
+
+				case GERBV_APTYPE_MACRO_POLYGON:
+					if (trans->scaleX == trans->scaleY) {
+						sam->parameter[POLYGON_CENTER_X]
+							*= trans->scaleX;
+						sam->parameter[POLYGON_CENTER_Y]
+							*= trans->scaleX;
+						sam->parameter[POLYGON_DIAMETER]
+							*= trans->scaleX;
+					} else {
+						/* TODO: make outline macro */
+						err_scale_poly_macro++;
+						break;
+					}
+
+					sam->parameter[POLYGON_ROTATION] +=
+						RAD2DEG(trans->rotation);
+					break;
+
+				case GERBV_APTYPE_MACRO_MOIRE:
+					if (trans->scaleX == trans->scaleY) {
+						sam->parameter[MOIRE_CENTER_X]
+							*= trans->scaleX;
+						sam->parameter[MOIRE_CENTER_Y]
+							*= trans->scaleX;
+						sam->parameter[MOIRE_OUTSIDE_DIAMETER]
+							*= trans->scaleX;
+						sam->parameter[MOIRE_CIRCLE_THICKNESS]
+							*= trans->scaleX;
+						sam->parameter[MOIRE_GAP_WIDTH]
+							*= trans->scaleX;
+						sam->parameter[MOIRE_CROSSHAIR_THICKNESS]
+							*= trans->scaleX;
+						sam->parameter[MOIRE_CROSSHAIR_LENGTH]
+							*= trans->scaleX;
+					} else {
+						err_scale_moire_macro++;
+						break;
+					}
+
+					sam->parameter[MOIRE_ROTATION] +=
+						RAD2DEG(trans->rotation);
+					break;
+
+				case GERBV_APTYPE_MACRO_THERMAL:
+					if (trans->scaleX == trans->scaleY) {
+						sam->parameter[THERMAL_CENTER_X]
+							*= trans->scaleX;
+						sam->parameter[THERMAL_CENTER_Y]
+							*= trans->scaleX;
+						sam->parameter[THERMAL_INSIDE_DIAMETER]
+							*= trans->scaleX;
+						sam->parameter[THERMAL_OUTSIDE_DIAMETER]
+							*= trans->scaleX;
+						sam->parameter[THERMAL_CROSSHAIR_THICKNESS]
+							*= trans->scaleX;
+					} else {
+						err_scale_thermo_macro++;
+						break;
+					}
+
+					sam->parameter[THERMAL_ROTATION] +=
+						RAD2DEG(trans->rotation);
+					break;
+
+				default:
+					/* TODO: free aper if it is skipped (i.e. unused)? */
+					err_unknown_macro_aperture++;
+				}
+			}
+
+			trans_apers[newNet->aperture] = ++aper_last_id;
+			destImage->aperture[aper_last_id] = aper;
+			newNet->aperture = aper_last_id;
+
+			break;
+		default:
+			err_unknown_aperture++;
+		}
+	}
+
+	if (err_rotate_rect)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't rotate %u rectangular aperture to %.2f "
+			"degrees (non 90 multiply)!",
+			"Can't rotate %u rectangular apertures to %.2f "
+			"degrees (non 90 multiply)!", err_rotate_rect),
+			err_rotate_rect, RAD2DEG(trans->rotation));
+
+	if (err_scale_line_macro)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't scale %u line macro!",
+			"Can't scale %u line macros!",
+			err_scale_line_macro), err_scale_line_macro);
+
+	if (err_scale_poly_macro)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't scale %u polygon macro!",
+			"Can't scale %u polygon macros!",
+			err_scale_poly_macro), err_scale_poly_macro);
+
+	if (err_scale_thermo_macro)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't scale %u thermal macro!",
+			"Can't scale %u thermal macros!",
+			err_scale_poly_macro), err_scale_poly_macro);
+
+	if (err_scale_moire_macro)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't scale %u moire macro!",
+			"Can't scale %u moire macros!",
+			err_scale_poly_macro), err_scale_poly_macro);
+
+	if (err_rotate_oval)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't rotate %u oval aperture to %.2f "
+			"degrees (non 90 multiply)!",
+			"Can't rotate %u oval apertures to %.2f "
+			"degrees (non 90 multiply)!", err_rotate_oval),
+			err_rotate_oval, RAD2DEG(trans->rotation));
+
+	if (err_scale_circle)
+		GERB_COMPILE_ERROR(ngettext(
+			"Can't scale %u circle aperture to ellipse!",
+			"Can't scale %u circle apertures to ellipse!",
+			err_scale_circle), err_scale_circle);
+
+	if (err_unknown_aperture)
+		GERB_COMPILE_ERROR(ngettext(
+			"Skipped %u aperture with unknown type!",
+			"Skipped %u apertures with unknown type!",
+			err_unknown_aperture), err_unknown_aperture);
+
+	if (err_unknown_macro_aperture)
+		GERB_COMPILE_ERROR(ngettext(
+			"Skipped %u macro aperture!",
+			"Skipped %u macro apertures!",
+			err_unknown_macro_aperture),
+				err_unknown_macro_aperture);
+
+	g_free (trans_apers);
 }
 
 gint
@@ -636,23 +1014,6 @@ gerbv_image_delete_net (gerbv_net_t *currentNet) {
 	currentNet->interpolation = GERBV_INTERPOLATION_DELETED;
 }
 
-void
-gerbv_image_delete_selected_nets (gerbv_image_t *sourceImage, GArray *selectedNodeArray) {
-	int i;
-	gerbv_net_t *currentNet;
-    
-	for (currentNet = sourceImage->netlist; currentNet; currentNet = currentNet->next){	
-		for (i=0; i<selectedNodeArray->len; i++){
-			gerbv_selection_item_t sItem = g_array_index (selectedNodeArray,
-				gerbv_selection_item_t, i);
-			if (sItem.net == currentNet) {
-				gerbv_image_delete_net (currentNet);
-				
-			}
-		}
-	}
-}
-
 void
 gerbv_image_create_rectangle_object (gerbv_image_t *image, gdouble coordinateX,
 		gdouble coordinateY, gdouble width, gdouble height) {
@@ -770,10 +1131,10 @@ gerbv_image_create_arc_object (gerbv_image_t *image, gdouble centerX, gdouble ce
 	currentNet->interpolation = GERBV_INTERPOLATION_CCW_CIRCULAR;
 	currentNet->aperture_state = GERBV_APERTURE_STATE_ON;
 	currentNet->aperture = apertureIndex;
-	currentNet->start_x = centerX + (cos(startAngle*M_PI/180) * radius);
-	currentNet->start_y = centerY + (sin(startAngle*M_PI/180) * radius);
-	currentNet->stop_x = centerX + (cos(endAngle*M_PI/180) * radius);
-	currentNet->stop_y = centerY + (sin(endAngle*M_PI/180) * radius);;
+	currentNet->start_x = centerX + (cos(DEG2RAD(startAngle)) * radius);
+	currentNet->start_y = centerY + (sin(DEG2RAD(startAngle)) * radius);
+	currentNet->stop_x = centerX + (cos(DEG2RAD(endAngle)) * radius);
+	currentNet->stop_y = centerY + (sin(DEG2RAD(endAngle)) * radius);
 	currentNet->cirseg = g_new0 (gerbv_cirseg_t,1);
 	*(currentNet->cirseg) = cirSeg;
 	
@@ -781,11 +1142,11 @@ gerbv_image_create_arc_object (gerbv_image_t *image, gdouble centerX, gdouble ce
 	gint i, steps = abs(angleDiff);
 	for (i=0; i<=steps; i++){
 		gdouble tempX = currentNet->cirseg->cp_x + currentNet->cirseg->width / 2.0 *
-				 cos ((currentNet->cirseg->angle1 +
-				 (angleDiff * i) / steps)*M_PI/180);
+				cos (DEG2RAD(currentNet->cirseg->angle1 +
+						(i*angleDiff)/steps));
 		gdouble tempY = currentNet->cirseg->cp_y + currentNet->cirseg->width / 2.0 *
-				 sin ((currentNet->cirseg->angle1 +
-				 (angleDiff * i) / steps)*M_PI/180);
+				sin (DEG2RAD(currentNet->cirseg->angle1 +
+						(i*angleDiff)/steps));
 		gerber_update_min_and_max (&currentNet->boundingBox,
 			       tempX, tempY, 
 			       lineWidth/2,lineWidth/2,
@@ -868,7 +1229,8 @@ gerbv_image_reduce_area_of_selected_objects (GArray *selectionArray,
 		minY = HUGE_VAL;
 		maxY = -HUGE_VAL;
 		
-		if (currentNet->interpolation == GERBV_INTERPOLATION_PAREA_START) {
+		switch (currentNet->interpolation) {
+		case GERBV_INTERPOLATION_PAREA_START:
 			/* if it's a polygon, just determine the overall area of it and delete it */
 			currentNet->interpolation = GERBV_INTERPOLATION_DELETED;
 			
@@ -886,25 +1248,30 @@ gerbv_image_reduce_area_of_selected_objects (GArray *selectionArray,
 					maxY = currentNet->stop_y;
 			}
 			currentNet->interpolation = GERBV_INTERPOLATION_DELETED;
-		}
-		else if ((currentNet->interpolation == GERBV_INTERPOLATION_x10) ||
-				(currentNet->interpolation == GERBV_INTERPOLATION_LINEARx01) ||
-				(currentNet->interpolation == GERBV_INTERPOLATION_LINEARx001) ||
-				(currentNet->interpolation == GERBV_INTERPOLATION_LINEARx1)) {
+
+			break;
+
+		case GERBV_INTERPOLATION_LINEARx1:
+		case GERBV_INTERPOLATION_LINEARx10:
+		case GERBV_INTERPOLATION_LINEARx01:
+		case GERBV_INTERPOLATION_LINEARx001: {
 			gdouble dx=0,dy=0;
+			gerbv_aperture_t *apert =
+				image->aperture[currentNet->aperture];
+
 			/* figure out the overall size of this element */
-			switch (image->aperture[currentNet->aperture]->type) {
-				case GERBV_APTYPE_CIRCLE :
-				case GERBV_APTYPE_OVAL :
-				case GERBV_APTYPE_POLYGON :
-					dx = dy = image->aperture[currentNet->aperture]->parameter[0];
-					break;
-				case GERBV_APTYPE_RECTANGLE :
-					dx = (image->aperture[currentNet->aperture]->parameter[0]/ 2);
-					dy = (image->aperture[currentNet->aperture]->parameter[1]/ 2);
-					break;
-				default :
-					break;
+			switch (apert->type) {
+			case GERBV_APTYPE_CIRCLE :
+			case GERBV_APTYPE_OVAL :
+			case GERBV_APTYPE_POLYGON :
+				dx = dy = apert->parameter[0];
+				break;
+			case GERBV_APTYPE_RECTANGLE :
+				dx = apert->parameter[0]/2;
+				dy = apert->parameter[1]/2;
+				break;
+			default :
+				break;
 			}
 			if (currentNet->start_x-dx < minX)
 				minX = currentNet->start_x-dx;
@@ -926,10 +1293,14 @@ gerbv_image_reduce_area_of_selected_objects (GArray *selectionArray,
 			
 			/* finally, delete node */
 			currentNet->interpolation = GERBV_INTERPOLATION_DELETED;
+
+			break;
 		}
-		/* we don't current support arcs */
-		else
+
+		default:
+			/* we don't current support arcs */
 			return FALSE;
+		}
 		
 		/* create new structures */
 		gerbv_image_create_window_pane_objects (image, minX, minY, maxX - minX, maxY - minY,
