@@ -27,44 +27,36 @@
     \ingroup libgerbv
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "gerbv.h"
 
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#ifdef HAVE_LIBGEN_H
-#include <libgen.h> /* dirname */
-#endif
 #include <errno.h>
 
+#ifdef HAVE_LIBGEN_H
+# include <libgen.h> /* dirname */
+#endif
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+# include <string.h>
 #endif
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+# include <unistd.h>
 #endif
 
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
-
 #ifdef HAVE_GETOPT_H
-  #include <getopt.h>
+# include <getopt.h>
 #endif
 
 #include <pango/pango.h>
 
-#include <locale.h>
-
 #include "common.h"
-#include "gerbv.h"
 #include "gerber.h"
 #include "drill.h"
+#include "selection.h"
 
 #include "draw-gdk.h"
 #include "draw.h"
@@ -74,6 +66,57 @@
 /* DEBUG printing.  #define DEBUG 1 in config.h to use this fcn. */
 #define dprintf if(DEBUG) printf
 
+/** Return string name of gerbv_aperture_type_t aperture type. */
+const char *gerbv_aperture_type_name(gerbv_aperture_type_t type)
+{
+	/* These are the names of the valid apertures.  Please keep this in
+	 * sync with the gerbv_aperture_type_t enum defined in gerbv.h */
+	const char *names[] = {
+		N_("none"),
+		N_("circle"),
+		N_("rectangle"),
+		N_("oval"),		/* ovular (obround) aperture */
+		N_("polygon"),		/* polygon aperture */
+		N_("macro"),		/* RS274X macro */
+		N_("circle macro"),	/* RS274X circle macro */
+		N_("outline macro"),	/* RS274X outline macro */
+		N_("polygon macro"),	/* RS274X polygon macro */
+		N_("moire macro"),	/* RS274X moire macro */
+		N_("thermal macro"),	/* RS274X thermal macro */
+		N_("line20 macro"),	/* RS274X line (code 20) macro */
+		N_("line21 macro"),	/* RS274X line (code 21) macro */
+		N_("line22 macro"),	/* RS274X line (code 22) macro */
+	};
+
+	if (type >=0 && type < sizeof(names)/sizeof(names[0]))
+		return names[type];
+
+	return N_("<undefined>");
+}
+
+/** Return string name of gerbv_interpolation_t interpolation. */
+const char *gerbv_interpolation_name(gerbv_interpolation_t interp)
+{
+	/* These are the names of the interpolation method.  Please keep this
+	 * in sync with the gerbv_interpolation_t enum defined in gerbv.h */
+	const char *names[] = {
+		N_("1X linear"),
+		N_("10X linear"),
+		N_("0.1X linear"),
+		N_("0.01X linear"),
+		N_("CW circular"),
+		N_("CCW circular"),
+		N_("poly area start"),
+		N_("poly area stop"),
+		N_("deleted"),
+	};
+
+	if (interp >= 0 && interp < sizeof(names)/sizeof(names[0]))
+		return names[interp];
+
+	return N_("<undefined>");
+}
+
 #define NUMBER_OF_DEFAULT_COLORS 18
 #define NUMBER_OF_DEFAULT_TRANSFORMATIONS 20
 
@@ -144,13 +187,16 @@ gerbv_create_project (void) {
 
 /* ------------------------------------------------------------------ */
 void
-gerbv_destroy_project (gerbv_project_t *gerbvProject){
+gerbv_destroy_project (gerbv_project_t *gerbvProject)
+{
 	int i;
 	
 	/* destroy all the files attached to the project */
 	for(i = gerbvProject->last_loaded; i >= 0; i--) {
-		if (gerbvProject->file[i])
+		if (gerbvProject->file[i]) {
 			gerbv_destroy_fileinfo (gerbvProject->file[i]);
+			g_free(gerbvProject->file[i]);
+		}
 	}
 	/* destroy strings */
 	g_free (gerbvProject->path);
@@ -182,8 +228,8 @@ gerbv_open_layer_from_filename(gerbv_project_t *gerbvProject, gchar *filename)
   dprintf("Opening filename = %s\n", (gchar *) filename);
   
   if (gerbv_open_image(gerbvProject, filename, ++gerbvProject->last_loaded, FALSE, NULL, 0, TRUE) == -1) {
-    GERB_MESSAGE(_("could not read %s[%d]"), (gchar *) filename,
-		 gerbvProject->last_loaded);
+    GERB_COMPILE_WARNING(_("Could not read \"%s\" (loaded %d)"),
+		    (gchar *) filename, gerbvProject->last_loaded);
     gerbvProject->last_loaded--;
   } else {
     idx_loaded = gerbvProject->last_loaded;
@@ -201,8 +247,8 @@ gerbv_open_layer_from_filename_with_color(gerbv_project_t *gerbvProject, gchar *
   dprintf("Opening filename = %s\n", (gchar *) filename);
   
   if (gerbv_open_image(gerbvProject, filename, ++gerbvProject->last_loaded, FALSE, NULL, 0, TRUE) == -1) {
-    GERB_MESSAGE(_("could not read %s[%d]"), (gchar *) filename,
-		 gerbvProject->last_loaded);
+    GERB_COMPILE_WARNING(_("Could not read \"%s\" (loaded %d)"),
+		    (gchar *) filename, gerbvProject->last_loaded);
     gerbvProject->last_loaded--;
   } else {
     idx_loaded = gerbvProject->last_loaded;
@@ -218,20 +264,52 @@ gerbv_open_layer_from_filename_with_color(gerbv_project_t *gerbvProject, gchar *
 gboolean 
 gerbv_save_layer_from_index(gerbv_project_t *gerbvProject, gint index, gchar *filename) 
 {
-    if (strcmp (gerbvProject->file[index]->image->info->type,"RS274-X (Gerber) File")==0) {
-	gerbv_export_rs274x_file_from_image (filename, gerbvProject->file[index]->image,
-		&gerbvProject->file[index]->transform);
-    }
-    else if (strcmp (gerbvProject->file[index]->image->info->type,"Excellon Drill File")==0) {
-	gerbv_export_drill_file_from_image (filename, gerbvProject->file[index]->image,
-		&gerbvProject->file[index]->transform);
-    }
-    else {
-	return FALSE;
-    }
-    gerbvProject->file[index]->layer_dirty = FALSE;
-    return TRUE;
-} /* gerbv_save_project_from_filename */
+	gerbv_fileinfo_t *file = gerbvProject->file[index];
+	gerbv_user_transformation_t *trans = &file->transform;
+
+	switch (file->image->layertype) {
+	case GERBV_LAYERTYPE_RS274X:
+		if (trans) {
+			/* NOTE: mirrored file is not yet supported */
+			if (trans->mirrorAroundX || trans->mirrorAroundY) {
+				GERB_COMPILE_ERROR(_("Exporting mirrored file "
+							"is not supported!"));
+				return FALSE;
+			}
+
+			/* NOTE: inverted file is not yet supported */
+			if (trans->inverted) {
+				GERB_COMPILE_ERROR(_("Exporting inverted file "
+							"is not supported!"));
+				return FALSE;
+			}
+		}
+
+		gerbv_export_rs274x_file_from_image (filename,
+				file->image, trans);
+		break;
+
+	case GERBV_LAYERTYPE_DRILL:
+		if (trans) {
+			/* NOTE: inverted file is not yet supported */
+			if (trans->inverted) {
+				GERB_COMPILE_ERROR(_("Exporting inverted file "
+							"is not supported!"));
+				return FALSE;
+			}
+		}
+
+		gerbv_export_drill_file_from_image (filename,
+				file->image, trans);
+		break;
+	default:
+		return FALSE;
+	}
+
+	file->layer_dirty = FALSE;
+
+	return TRUE;
+}
 
 
 /* ------------------------------------------------------------------ */
@@ -325,8 +403,7 @@ gerbv_add_parsed_image_to_project (gerbv_project_t *gerbvProject, gerbv_image_t
 
     if (error) {
 	if (error & GERB_IMAGE_MISSING_NETLIST) {
-	    GERB_COMPILE_ERROR(_("Missing netlist - aborting file read\n"));
-	    GERB_COMPILE_ERROR("\n");
+	    GERB_COMPILE_ERROR(_("Missing netlist - aborting file read"));
 	    gerbv_destroy_image(parsed_image);
 	    return -1;
 	}
@@ -425,7 +502,8 @@ gerbv_open_image(gerbv_project_t *gerbvProject, char *filename, int idx, int rel
     
     fd = gerb_fopen(filename);
     if (fd == NULL) {
-	GERB_MESSAGE(_("Trying to open %s:%s\n"), filename, strerror(errno));
+	GERB_COMPILE_ERROR(_("Trying to open \"%s\": %s"),
+			filename, strerror(errno));
 	return -1;
     }
 
@@ -441,7 +519,7 @@ gerbv_open_image(gerbv_project_t *gerbvProject, char *filename, int idx, int rel
 
     if (gerber_is_rs274x_p(fd, &foundBinary)) {
 	dprintf("Found RS-274X file\n");
-	if ((!foundBinary || forceLoadFile)) {
+	if (!foundBinary || forceLoadFile) {
 		/* figure out the directory path in case parse_gerb needs to
 		 * load any include files */
 		gchar *currentLoadDirectory = g_path_get_dirname (filename);
@@ -450,19 +528,43 @@ gerbv_open_image(gerbv_project_t *gerbvProject, char *filename, int idx, int rel
 	}
     } else if(drill_file_p(fd, &foundBinary)) {
 	dprintf("Found drill file\n");
-	if ((!foundBinary || forceLoadFile))
+	if (!foundBinary || forceLoadFile)
 	    parsed_image = parse_drillfile(fd, attr_list, n_attr, reload);
 	
     } else if (pick_and_place_check_file_type(fd, &foundBinary)) {
 	dprintf("Found pick-n-place file\n");
-	if ((!foundBinary || forceLoadFile)) {
-		pick_and_place_parse_file_to_images(fd, &parsed_image, &parsed_image2);
+	if (!foundBinary || forceLoadFile) {
+		if (!reload) {
+			pick_and_place_parse_file_to_images(fd, &parsed_image, &parsed_image2);
+		} else {
+			switch (gerbvProject->file[idx]->image->layertype) {
+			case GERBV_LAYERTYPE_PICKANDPLACE_TOP:
+				/* Non NULL pointer is used as "not to reload" mark */
+				parsed_image2 = (void *)!NULL;
+				pick_and_place_parse_file_to_images(fd, &parsed_image, &parsed_image2);
+				parsed_image2 = NULL;
+				break;
+			case GERBV_LAYERTYPE_PICKANDPLACE_BOT:
+				/* Non NULL pointer is used as "not to reload" mark */
+				parsed_image2 = (void *)!NULL;
+				pick_and_place_parse_file_to_images(fd, &parsed_image2, &parsed_image);
+				parsed_image2 = NULL;
+				break;
+			default:
+				GERB_COMPILE_ERROR(_("%s: unknown pick-and-place board side to reload"), filename);
+			}
+		}
+			
 		isPnpFile = TRUE;
 	}
     } else if (gerber_is_rs274d_p(fd)) {
-	dprintf("Most likely found a RS-274D file...trying to open anyways");
-	g_warning(_("Most likely found a RS-274D file...trying to open anyways"));
-	if ((!foundBinary || forceLoadFile)) {
+	gchar *str = g_strdup_printf(_("Most likely found a RS-274D file "
+			"\"%s\" ... trying to open anyways\n"), filename);
+	dprintf(str);
+	g_warning(str);
+	g_free (str);
+
+	if (!foundBinary || forceLoadFile) {
 		/* figure out the directory path in case parse_gerb needs to
 		 * load any include files */
 		gchar *currentLoadDirectory = g_path_get_dirname (filename);
@@ -472,10 +574,11 @@ gerbv_open_image(gerbv_project_t *gerbvProject, char *filename, int idx, int rel
     } else {
 	/* This is not a known file */
 	dprintf("Unknown filetype");
-	GERB_COMPILE_ERROR(_("%s: Unknown file type.\n"), filename);
+	GERB_COMPILE_ERROR(_("%s: Unknown file type."), filename);
 	parsed_image = NULL;
     }
     
+    g_free(fd->filename);
     gerb_fclose(fd);
     if (parsed_image == NULL) {
 	return -1;
@@ -519,7 +622,8 @@ gerbv_create_rs274x_image_from_filename (gchar *filename){
 	
 	fd = gerb_fopen(filename);
 	if (fd == NULL) {
-		GERB_MESSAGE(_("Trying to open %s:%s\n"), filename, strerror(errno));
+		GERB_COMPILE_ERROR(_("Trying to open \"%s\": %s"),
+				filename, strerror(errno));
 		return NULL;
 	}
 	gchar *currentLoadDirectory = g_path_get_dirname (filename);
@@ -529,6 +633,12 @@ gerbv_create_rs274x_image_from_filename (gchar *filename){
 	return returnImage;
 }
 
+static inline int isnormal_or_zero(double x)
+{
+	int cl = fpclassify(x);
+	return cl == FP_NORMAL || cl == FP_ZERO;
+}
+
 /* ------------------------------------------------------------------ */
 void
 gerbv_render_get_boundingbox(gerbv_project_t *gerbvProject, gerbv_render_size_t *boundingbox)
@@ -539,7 +649,7 @@ gerbv_render_get_boundingbox(gerbv_project_t *gerbvProject, gerbv_render_size_t
 	gdouble minX, minY, maxX, maxY;
 	
 	for(i = 0; i <= gerbvProject->last_loaded; i++) {
-		if ((gerbvProject->file[i]) && (gerbvProject->file[i]->isVisible)){
+		if (gerbvProject->file[i] && gerbvProject->file[i]->isVisible){
 			
 			
 			info = gerbvProject->file[i]->image->info;
@@ -552,12 +662,13 @@ gerbv_render_get_boundingbox(gerbv_project_t *gerbvProject, gerbv_render_size_t
 			minY = info->min_y;
 			maxX = info->max_x;
 			maxY = info->max_y;
-			
-			if (!isnormal(minX)||!isnormal(minY)||!isnormal(maxX)||!isnormal(maxY)){
+
+			if (!isnormal_or_zero(minX) || !isnormal_or_zero(minY)
+			 || !isnormal_or_zero(maxX) || !isnormal_or_zero(maxY)) {
 				continue;
 			}
+
 			/* transform the bounding box based on the user transform */
-	
 			cairo_matrix_t fullMatrix;
 			cairo_matrix_init (&fullMatrix, 1, 0, 0, 1, 0, 0);
 
@@ -595,7 +706,8 @@ gerbv_render_get_boundingbox(gerbv_project_t *gerbvProject, gerbv_render_size_t
 
 /* ------------------------------------------------------------------ */
 void
-gerbv_render_zoom_to_fit_display (gerbv_project_t *gerbvProject, gerbv_render_info_t *renderInfo) {
+gerbv_render_zoom_to_fit_display (gerbv_project_t *gerbvProject, gerbv_render_info_t *renderInfo)
+{
 	gerbv_render_size_t bb;
 	double width, height;
 	double x_scale, y_scale;
@@ -614,7 +726,7 @@ gerbv_render_zoom_to_fit_display (gerbv_project_t *gerbvProject, gerbv_render_in
 		renderInfo->lowerLeftX = 0.0;
 		renderInfo->lowerLeftY = 0.0;
 		renderInfo->scaleFactorX = 200;
-		renderInfo->scaleFactorY = 200;
+		renderInfo->scaleFactorY = renderInfo->scaleFactorX;
 		return;
 	}
 	/*
@@ -626,11 +738,11 @@ gerbv_render_zoom_to_fit_display (gerbv_project_t *gerbvProject, gerbv_render_in
 	* Take the scale that fits both directions with some extra checks
 	*/
 	renderInfo->scaleFactorX = MIN(x_scale, y_scale);
+	renderInfo->scaleFactorX = MIN((gdouble)GERBV_SCALE_MAX,
+			renderInfo->scaleFactorX);
+	renderInfo->scaleFactorX = MAX((gdouble)GERBV_SCALE_MIN,
+			renderInfo->scaleFactorX);
 	renderInfo->scaleFactorY = renderInfo->scaleFactorX;
-	if (renderInfo->scaleFactorX < 1){
-	    renderInfo->scaleFactorX = 1;
-	    renderInfo->scaleFactorY = 1;
-	}
 	renderInfo->lowerLeftX = ((bb.left + bb.right) / 2.0) -
 		((double) renderInfo->displayWidth / 2.0 / renderInfo->scaleFactorX);
 	renderInfo->lowerLeftY = ((bb.top + bb.bottom) / 2.0) -
@@ -722,36 +834,48 @@ gerbv_render_to_pixmap_using_gdk (gerbv_project_t *gerbvProject, GdkPixmap *pixm
 			gdk_gc_set_clip_mask(gc, NULL);
 		}
 	}
-	/* render the selection group to the top of the output */
-	if ((selectionInfo) && (selectionInfo->type != GERBV_SELECTION_EMPTY)) {
+
+	/* Render the selection group to the top of the output */
+	if (selectionInfo && selectionInfo->selectedNodeArray
+	&& (selection_length (selectionInfo) != 0)) {
 		if (!selectionColor->pixel)
 	 		gdk_colormap_alloc_color(gdk_colormap_get_system(), selectionColor, FALSE, TRUE);
 
 		gdk_gc_set_foreground(gc, selectionColor);
 		gdk_gc_set_function(gc, GDK_COPY);
 		gdk_draw_rectangle(colorStamp, gc, TRUE, 0, 0, -1, -1);
-		
-		/* for now, assume everything in the selection buffer is from one image */
-		gerbv_image_t *matchImage;
-		int j;
-		if (selectionInfo->selectedNodeArray->len > 0) {
-			gerbv_selection_item_t sItem = g_array_index (selectionInfo->selectedNodeArray,
-					gerbv_selection_item_t, 0);
-			matchImage = (gerbv_image_t *) sItem.image;	
 
-			for(j = gerbvProject->last_loaded; j >= 0; j--) {
-				if ((gerbvProject->file[j]) && (gerbvProject->file[j]->image == matchImage)) {
-					draw_gdk_image_to_pixmap(&clipmask, gerbvProject->file[j]->image,
-						renderInfo->scaleFactorX, -(renderInfo->lowerLeftX * renderInfo->scaleFactorX),
-						(renderInfo->lowerLeftY * renderInfo->scaleFactorY) + renderInfo->displayHeight,
-						DRAW_SELECTIONS, selectionInfo,
-						renderInfo, gerbvProject->file[j]->transform);
-				}
+		gerbv_selection_item_t sItem;
+		gerbv_fileinfo_t *file;
+		int j;
+		guint k;
+
+		for (j = gerbvProject->last_loaded; j >= 0; j--) {
+			file = gerbvProject->file[j]; 
+			if (!file || (!gerbvProject->show_invisible_selection && !file->isVisible))
+				continue;
+
+			for (k = 0; k < selection_length (selectionInfo); k++) {
+				sItem = selection_get_item_by_index (selectionInfo, k);
+
+				if (file->image != sItem.image)
+					continue;
+
+				/* Have selected image(s) on this layer, draw it */
+				draw_gdk_image_to_pixmap(&clipmask, file->image,
+					renderInfo->scaleFactorX,
+					-(renderInfo->lowerLeftX * renderInfo->scaleFactorX),
+					(renderInfo->lowerLeftY * renderInfo->scaleFactorY) + renderInfo->displayHeight,
+					DRAW_SELECTIONS, selectionInfo,
+					renderInfo, file->transform);
+
+				gdk_gc_set_clip_mask(gc, clipmask);
+				gdk_gc_set_clip_origin(gc, 0, 0);
+				gdk_draw_drawable(pixmap, gc, colorStamp, 0, 0, 0, 0, -1, -1);
+				gdk_gc_set_clip_mask(gc, NULL);
+
+				break;
 			}
-			gdk_gc_set_clip_mask(gc, clipmask);
-			gdk_gc_set_clip_origin(gc, 0, 0);
-			gdk_draw_drawable(pixmap, gc, colorStamp, 0, 0, 0, 0, -1, -1);
-			gdk_gc_set_clip_mask(gc, NULL);
 		}
 	}
 
@@ -762,35 +886,64 @@ gerbv_render_to_pixmap_using_gdk (gerbv_project_t *gerbvProject, GdkPixmap *pixm
 
 /* ------------------------------------------------------------------ */
 void
-gerbv_render_all_layers_to_cairo_target_for_vector_output (gerbv_project_t *gerbvProject,
-		cairo_t *cr, gerbv_render_info_t *renderInfo) {
+gerbv_render_all_layers_to_cairo_target_for_vector_output (
+		gerbv_project_t *gerbvProject, cairo_t *cr,
+		gerbv_render_info_t *renderInfo)
+{
+	GdkColor *bg = &gerbvProject->background;
 	int i;
-	gerbv_render_cairo_set_scale_and_translation(cr, renderInfo);
-	/* don't paint background for vector output, since it isn't needed */
-	for(i = gerbvProject->last_loaded; i >= 0; i--) {
-		if (gerbvProject->file[i] && gerbvProject->file[i]->isVisible) {
+	double r, g, b;
 
-		    gerbv_render_layer_to_cairo_target_without_transforming(cr, gerbvProject->file[i], renderInfo, FALSE);
+	gerbv_render_cairo_set_scale_and_translation (cr, renderInfo);
+
+	/* Fill the background with the appropriate not white and not black
+	 * color for backward culpability. */ 
+	if ((bg->red != 0xffff || bg->green != 0xffff || bg->blue != 0xffff)
+	 && (bg->red != 0x0000 || bg->green != 0x0000 || bg->blue != 0x0000)) {
+		r = (double) bg->red/G_MAXUINT16;
+		g = (double) bg->green/G_MAXUINT16;
+		b = (double) bg->blue/G_MAXUINT16;
+		cairo_set_source_rgba (cr, r, g, b, 1);
+		cairo_paint (cr);
+
+		/* Set cairo user data with background color information, to be
+		 * used for clear color. */
+		cairo_set_user_data (cr, (cairo_user_data_key_t *)0, &r, NULL);
+		cairo_set_user_data (cr, (cairo_user_data_key_t *)1, &g, NULL);
+		cairo_set_user_data (cr, (cairo_user_data_key_t *)2, &b, NULL);
+	}
+
+	for (i = gerbvProject->last_loaded; i >= 0; i--) {
+		if (gerbvProject->file[i] && gerbvProject->file[i]->isVisible) {
+			gerbv_render_layer_to_cairo_target_without_transforming(
+					cr, gerbvProject->file[i],
+					renderInfo, FALSE);
 		}
 	}
 }
 
 /* ------------------------------------------------------------------ */
 void
-gerbv_render_all_layers_to_cairo_target (gerbv_project_t *gerbvProject, cairo_t *cr,
-			gerbv_render_info_t *renderInfo) {
+gerbv_render_all_layers_to_cairo_target (gerbv_project_t *gerbvProject,
+		cairo_t *cr, gerbv_render_info_t *renderInfo)
+{
 	int i;
-	/* fill the background with the appropriate color */
-	cairo_set_source_rgba (cr, (double) gerbvProject->background.red/G_MAXUINT16,
-		(double) gerbvProject->background.green/G_MAXUINT16,
-		(double) gerbvProject->background.blue/G_MAXUINT16, 1);
+
+	/* Fill the background with the appropriate color. */
+	cairo_set_source_rgba (cr,
+			(double) gerbvProject->background.red/G_MAXUINT16,
+			(double) gerbvProject->background.green/G_MAXUINT16,
+			(double) gerbvProject->background.blue/G_MAXUINT16, 1);
 	cairo_paint (cr);
-	for(i = gerbvProject->last_loaded; i >= 0; i--) {
+
+	for (i = gerbvProject->last_loaded; i >= 0; i--) {
 		if (gerbvProject->file[i] && gerbvProject->file[i]->isVisible) {
 			cairo_push_group (cr);
-			gerbv_render_layer_to_cairo_target (cr, gerbvProject->file[i], renderInfo);
+			gerbv_render_layer_to_cairo_target (cr,
+					gerbvProject->file[i], renderInfo);
 			cairo_pop_group_to_source (cr);
-			cairo_paint_with_alpha (cr, (double) gerbvProject->file[i]->alpha/G_MAXUINT16);
+			cairo_paint_with_alpha (cr, (double)
+					gerbvProject->file[i]->alpha/G_MAXUINT16);
 		}
 	}
 }
@@ -877,7 +1030,7 @@ gerbv_attribute_dup (gerbv_HID_Attribute *attributeList, int n_attr)
 
   nl = (gerbv_HID_Attribute *) malloc (n_attr * sizeof (gerbv_HID_Attribute));
   if (nl == NULL) {
-    fprintf (stderr, _("%s():  malloc failed\n"), __FUNCTION__);
+    fprintf (stderr, "malloc failed in %s()\n", __FUNCTION__);
     exit (1);
   }
 
@@ -901,4 +1054,63 @@ gerbv_attribute_dup (gerbv_HID_Attribute *attributeList, int n_attr)
   return nl;
 }
 
+gerbv_fileinfo_t *
+gerbv_get_fileinfo_for_image(const gerbv_image_t *image,
+		const gerbv_project_t *project)
+{
+	int i;
+
+	for (i = 0; i <= project->last_loaded; i++) {
+		if (project->file[i]->image == image)
+			return project->file[i];
+	}
+
+	return NULL;	
+}
+
+inline void
+gerbv_rotate_coord(double *x, double *y, double rad)
+{
+	double x0 = *x;
+
+	*x = x0*cos(rad) - *y*sin(rad);
+	*y = x0*sin(rad) + *y*cos(rad);
+}
+
+void
+gerbv_transform_coord(double *x, double *y,
+			const gerbv_user_transformation_t *trans)
+{
+
+	*x = trans->scaleX * *x;
+	*y = trans->scaleY * *y;
+
+	gerbv_rotate_coord(x, y, trans->rotation);
+
+	if (trans->mirrorAroundY)
+		*x = -*x;
+
+	if (trans->mirrorAroundX)
+		*y = -*y;
+
+	*x += trans->translateX;
+	*y += trans->translateY;
+}
+
+int
+gerbv_transform_coord_for_image(double *x, double *y,
+		const gerbv_image_t *image, const gerbv_project_t *project)
+{
+	gerbv_fileinfo_t *fileinfo =
+		gerbv_get_fileinfo_for_image(image, project);
+
+	if (fileinfo == NULL) {
+		dprintf("%s(): NULL fileinfo\n", __func__);
+		return -1;
+	}
+
+	gerbv_transform_coord(x, y, &fileinfo->transform);
+
+	return 0;
+}
 
