@@ -24,61 +24,28 @@
     \ingroup libgerbv
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <assert.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <errno.h>
-#include <sys/stat.h>
-#include <glib.h>
-#include <gtk/gtk.h> /* What's this for? */
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-
-#include <ctype.h>
-#include <locale.h>
-
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif /* HAVE_GETOPT_H */
-
-#include <assert.h>
-
 #include "gerbv.h"
-#include "gerber.h"
-#include "main.h"
-#ifdef RENDER_USING_GDK
-  #include "draw-gdk.h"
-#else
-  #include "draw.h"
-#endif
 
+#include <assert.h>
+#include <ctype.h>
+#include <math.h>
+#include <stdlib.h>
+
+#include "gerber.h"
 #include "common.h"
 #include "csv.h"
 #include "pick-and-place.h"
 
-#undef max
-#define max(a,b) ((a) > (b) ? (a) : (b))
-#undef min
-#define min(a,b) ((a) < (b) ? (a) : (b))
-
-/* DEBUG printing.  #define DEBUG 1 in config.h to use this fcn. */
-#define dprintf if(DEBUG) printf
+static gerbv_net_t *pnp_new_net(gerbv_net_t *net); 
+static void pnp_reset_bbox (gerbv_net_t *net);
+static void pnp_init_net(gerbv_net_t *net, gerbv_image_t *image,
+		const char *label,
+		gerbv_aperture_state_t apert_state,
+		gerbv_interpolation_t interpol);
 
 void gerb_transf_free(gerbv_transf_t *transf)
 {
     g_free(transf);
-    
 }
 
 
@@ -97,12 +64,11 @@ void gerb_transf_reset(gerbv_transf_t* transf)
 gerbv_transf_t* gerb_transf_new(void)
 {
     gerbv_transf_t *transf;
-    
-    transf = g_malloc(sizeof(gerbv_transf_t));
-    gerb_transf_reset(transf);
-    return transf;
-    
 
+    transf = g_new(gerbv_transf_t, 1);
+    gerb_transf_reset(transf);
+
+    return transf;
 } /* gerb_transf_new */
 
 
@@ -160,37 +126,47 @@ pick_and_place_reset_bounding_box (gerbv_net_t *net) {
 	net->boundingBox.top = HUGE_VAL;
 }
 
-//! Parses a string representing float number with a unit, default is mil
-static double 
-pick_and_place_get_float_unit(char *str)
+/* Parses a string representing float number with a unit.
+ * Default unit can be specified with def_unit. */
+static double
+pick_and_place_get_float_unit(const char *str, const char *def_unit)
 {
     double x = 0.0;
-    char unit[41];
-    extern gerbv_screen_t screen;
+    char unit_str[41] = {0,};
+    const char *unit = unit_str;
 
     /* float, optional space, optional unit mm,cm,in,mil */
-    sscanf(str, "%lf %40s", &x, unit);
-    if(strstr(unit,"in")) {
-	;
-    } else if(strstr(unit, "mil")) {
-	x /= 1000;
-    } else if(strstr(unit, "cm")) {
-	x /= 2.54;
-    } else if(strstr(unit, "mm")) {
+    sscanf(str, "%lf %40s", &x, unit_str);
+
+    if (unit_str[0] == '\0')
+	unit = def_unit;
+
+    /* NOTE: in order of comparability,
+     * i.e. "mm" before "m", as "m" will match "mm" */
+    if (strstr(unit, "mm")) {
 	x /= 25.4;
-    } else { /* default to GUI units */
-	switch (screen.unit) {
-	case GERBV_INS:
-	    break;
-	case GERBV_MILS:
-	    x /= 1000;
-	    break;
-	case GERBV_MMS:
-	    x /= 25.4;
-	    break;
-	default:
-	    assert(screen.unit);
-	}
+    } else if (strstr(unit, "in")) {
+	/* NOTE: "in" is without scaling. */
+    } else if (strstr(unit, "cmil")) {
+	x /= 1e5;
+    } else if (strstr(unit, "dmil")) {
+	x /= 1e4;
+    } else if (strstr(unit, "mil")) {
+	x /= 1e3;
+    } else if (strstr(unit, "km")) {
+	x /= 25.4/1e6;
+    } else if (strstr(unit, "dm")) {
+	x /= 25.4/100;
+    } else if (strstr(unit, "cm")) {
+	x /= 25.4/10;
+    } else if (strstr(unit, "um")) {
+	x /= 25.4*1e3;
+    } else if (strstr(unit, "nm")) {
+	x /= 25.4*1e6;
+    } else if (strstr(unit,  "m")) {
+	x /= 25.4/1e3;
+    } else { /* default to "mil" */
+	x /= 1e3;
     }
 
     return x;
@@ -254,10 +230,13 @@ pick_and_place_parse_file(gerb_file_t *fd)
     int           ret;
     char          *row[12];
     char          buf[MAXL+2], buf0[MAXL+2];
+    char          def_unit[41] = {0,};
     double        tmp_x, tmp_y;
     gerbv_transf_t *tr_rot = gerb_transf_new();
     GArray 	*pnpParseDataArray = g_array_new (FALSE, FALSE, sizeof(PnpPartData));
     gboolean foundValidDataRow = FALSE;
+    /* Unit declaration for "PcbXY Version 1.0" files as exported by pcb */
+    const char *def_unit_prefix = "# X,Y in ";
     
     /*
      * many locales redefine "." as "," and so on, so sscanf has problems when
@@ -283,6 +262,9 @@ pick_and_place_parse_file(gerb_file_t *fd)
 	if(len >= 0 && buf[len] == '\r') {
 	    buf[len--] = 0;
 	}
+	if (0 == strncmp(buf, def_unit_prefix, strlen(def_unit_prefix))) {
+	    sscanf(&buf[strlen(def_unit_prefix)], "%40s.", def_unit);
+	}
 	if (len <= 11)  {  //lets check a minimum length of 11
 	    continue;
 	}
@@ -340,12 +322,12 @@ pick_and_place_parse_file(gerb_file_t *fd)
 	    /*
 	      gchar* g_convert(const gchar *str, gssize len, const gchar *to_codeset, const gchar *from_codeset, gsize *bytes_read, gsize *bytes_written, GError **error);
 	    */
-	    pnpPartData.mid_x = pick_and_place_get_float_unit(row[2]);
-	    pnpPartData.mid_y = pick_and_place_get_float_unit(row[3]);
-	    pnpPartData.ref_x = pick_and_place_get_float_unit(row[4]);
-	    pnpPartData.ref_y = pick_and_place_get_float_unit(row[5]);
-	    pnpPartData.pad_x = pick_and_place_get_float_unit(row[6]);
-	    pnpPartData.pad_y = pick_and_place_get_float_unit(row[7]);
+	    pnpPartData.mid_x = pick_and_place_get_float_unit(row[2], def_unit);
+	    pnpPartData.mid_y = pick_and_place_get_float_unit(row[3], def_unit);
+	    pnpPartData.ref_x = pick_and_place_get_float_unit(row[4], def_unit);
+	    pnpPartData.ref_y = pick_and_place_get_float_unit(row[5], def_unit);
+	    pnpPartData.pad_x = pick_and_place_get_float_unit(row[6], def_unit);
+	    pnpPartData.pad_y = pick_and_place_get_float_unit(row[7], def_unit);
 	    /* This line causes segfault if we accidently starts parsing 
 	     * a gerber file. It is crap crap crap */
 	    if (row[9])
@@ -358,8 +340,8 @@ pick_and_place_parse_file(gerb_file_t *fd)
 	    snprintf (pnpPartData.designator, sizeof(pnpPartData.designator)-1, "%s", row[0]);
 	    snprintf (pnpPartData.footprint, sizeof(pnpPartData.footprint)-1, "%s", row[1]);		
 	    snprintf (pnpPartData.layer, sizeof(pnpPartData.layer)-1, "%s", row[6]);	
-	    pnpPartData.mid_x = pick_and_place_get_float_unit(row[3]);
-	    pnpPartData.mid_y = pick_and_place_get_float_unit(row[4]);
+	    pnpPartData.mid_x = pick_and_place_get_float_unit(row[3], def_unit);
+	    pnpPartData.mid_y = pick_and_place_get_float_unit(row[4], def_unit);
 	    pnpPartData.pad_x = pnpPartData.mid_x + 0.03;
 	    pnpPartData.pad_y = pnpPartData.mid_y + 0.03;
 	    sscanf(row[5], "%lf", &pnpPartData.rotation); // no units, always deg
@@ -385,7 +367,7 @@ pick_and_place_parse_file(gerb_file_t *fd)
 	    pnpPartData.shape = PART_SHAPE_RECTANGLE;
 	} else { 
 	    gerb_transf_reset(tr_rot);
-	    gerb_transf_rotate(tr_rot, -pnpPartData.rotation * M_PI/180);/* rotate it back to get dimensions */
+	    gerb_transf_rotate(tr_rot, -DEG2RAD(pnpPartData.rotation));/* rotate it back to get dimensions */
 	    gerb_transf_apply( pnpPartData.pad_x -  pnpPartData.mid_x, 
 			       pnpPartData.pad_y - pnpPartData.mid_y, tr_rot, &tmp_x, &tmp_y);
 	    if ((fabs(tmp_y) > fabs(tmp_x/100)) && (fabs(tmp_x) > fabs(tmp_y/100))){
@@ -446,7 +428,7 @@ pick_and_place_check_file_type(gerb_file_t *fd, gboolean *returnFoundBinary)
 
     buf = malloc(MAXL);
     if (buf == NULL)
-	GERB_FATAL_ERROR(_("malloc buf failed while checking for pick-place file.\n"));
+	GERB_FATAL_ERROR("malloc buf failed in %s()", __FUNCTION__);
 
     while (fgets(buf, MAXL, fd->fd) != NULL) {
 	len = strlen(buf);
@@ -561,7 +543,8 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
     gerbv_transf_t *tr_rot = gerb_transf_new();
     gerbv_drill_stats_t *stats;  /* Eventually replace with pick_place_stats */
     gboolean foundElement = FALSE;
-    
+    const double draw_width = 0.01;
+
     /* step through and make sure we have an element on the layer before
        we actually create a new image for it and fill it */
     for (i = 0; i < parsedPickAndPlaceData->len; i++) {
@@ -579,49 +562,51 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 
     image = gerbv_create_image(image, "Pick and Place (X-Y) File");
     if (image == NULL) {
-	GERB_FATAL_ERROR(_("malloc image failed\n"));
+	GERB_FATAL_ERROR("malloc image failed in %s()", __FUNCTION__);
     }
-	
-    image->format = (gerbv_format_t *)g_malloc(sizeof(gerbv_format_t));
+
+    image->format = g_new0(gerbv_format_t, 1);
     if (image->format == NULL) {
-	GERB_FATAL_ERROR(_("malloc format failed\n"));
+	GERB_FATAL_ERROR("malloc format failed in %s()", __FUNCTION__);
     }
-    memset((void *)image->format, 0, sizeof(gerbv_format_t));
-    
-    image->layertype = GERBV_LAYERTYPE_PICKANDPLACE;
+
+    /* Separate top/bot layer type is needed for reload purpose */
+    if (boardSide == 1)
+	    image->layertype = GERBV_LAYERTYPE_PICKANDPLACE_TOP;
+    else
+	    image->layertype = GERBV_LAYERTYPE_PICKANDPLACE_BOT;
+
     stats = gerbv_drill_stats_new();
     if (stats == NULL)
-        GERB_FATAL_ERROR(_("malloc pick_place_stats failed\n"));
+        GERB_FATAL_ERROR("malloc pick_place_stats failed in %s()",
+			__FUNCTION__);
     image->drill_stats = stats;
 
 
     curr_net = image->netlist;
     curr_net->layer = image->layers;
     curr_net->state = image->states;
-    pick_and_place_reset_bounding_box (curr_net);	
+    pnp_reset_bbox (curr_net);	
     image->info->min_x = HUGE_VAL;
     image->info->min_y = HUGE_VAL;
     image->info->max_x = -HUGE_VAL;
     image->info->max_y = -HUGE_VAL;
 
-    image->aperture[0] = (gerbv_aperture_t *)g_malloc(sizeof(gerbv_aperture_t));
-    memset((void *) image->aperture[0], 0, sizeof(gerbv_aperture_t));
+    image->aperture[0] = g_new0(gerbv_aperture_t, 1);
+    assert(image->aperture[0] != NULL);
     image->aperture[0]->type = GERBV_APTYPE_CIRCLE;
     image->aperture[0]->amacro = NULL;
-    image->aperture[0]->parameter[0] = 0.01;
+    image->aperture[0]->parameter[0] = draw_width;
     image->aperture[0]->nuf_parameters = 1;
 
     for (i = 0; i < parsedPickAndPlaceData->len; i++) {
 	PnpPartData partData = g_array_index(parsedPickAndPlaceData, PnpPartData, i);
 	float radius,labelOffset;  
-	                       
-	curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	curr_net = curr_net->next;
-	
-	assert(curr_net);
-	memset((void *)curr_net, 0, sizeof(gerbv_net_t));
+
+	curr_net = pnp_new_net(curr_net);
 	curr_net->layer = image->layers;
 	curr_net->state = image->states;
+
 	if ((partData.rotation > 89) && (partData.rotation < 91))
 		labelOffset = fabs(partData.length/2);
 	else if ((partData.rotation > 179) && (partData.rotation < 181))
@@ -635,32 +620,25 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 	else if ((partData.rotation > -271) && (partData.rotation < -269))
 		labelOffset = fabs(partData.length/2);
 	else labelOffset = fabs(partData.width/2);
-	
-	partData.rotation *= M_PI/180; /* convert deg to rad */
+
+	partData.rotation = DEG2RAD(partData.rotation);
+
 	/* check if the entry is on the specified layer */
 	if ((boardSide == 0) && !((partData.layer[0]=='b') || (partData.layer[0]=='B')))
 		continue;
 	if ((boardSide == 1) && !((partData.layer[0]=='t') || (partData.layer[0]=='T')))
 		continue;
 
-	/* this first net is just a label holder, so calculate the lower
-	      left location to line up above the element */
+	curr_net = pnp_new_net(curr_net);
+	pnp_init_net(curr_net, image, partData.designator,
+			GERBV_APERTURE_STATE_OFF,
+			GERBV_INTERPOLATION_LINEARx1);
 
+	/* First net of PNP is just a label holder, so calculate the lower left
+	 * location to line up above the element */
 	curr_net->start_x = curr_net->stop_x = partData.mid_x;
-	curr_net->start_y = curr_net->stop_y = partData.mid_y + labelOffset + 0.01;   
-	curr_net->aperture = 0;
-	curr_net->aperture_state = GERBV_APERTURE_STATE_OFF;
-	curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	curr_net->layer = image->layers;
-	curr_net->state = image->states;
-	pick_and_place_reset_bounding_box (curr_net);
-
-	/* assign a label to this first draw primitive, in case we want
-	* to render some text next to the mark
-	*/
-	if (strlen (partData.designator) > 0) {
-	curr_net->label = g_string_new (partData.designator);
-	}
+	curr_net->start_y = curr_net->stop_y =
+		partData.mid_y + labelOffset + draw_width;   
 
 	gerb_transf_reset(tr_rot);
 	gerb_transf_shift(tr_rot, partData.mid_x, partData.mid_y);
@@ -670,80 +648,53 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 	    (partData.shape == PART_SHAPE_STD)) {
 	    // TODO: draw rectangle length x width taking into account rotation or pad x,y
 
+	    curr_net = pnp_new_net(curr_net);
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_LINEARx1);
 
-	    
-	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	    curr_net = curr_net->next;
-	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    gerb_transf_apply(partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
 	    gerb_transf_apply(-partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->stop_x, &curr_net->stop_y);
 	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    pick_and_place_reset_bounding_box (curr_net);
-	    
-	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	    curr_net = curr_net->next;
-	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
-	    
+/* TODO: write unifying function */
+
+	    curr_net = pnp_new_net(curr_net);
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_LINEARx1);
+
 	    gerb_transf_apply(-partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
 	    gerb_transf_apply(-partData.length/2, -partData.width/2, tr_rot, 
 			      &curr_net->stop_x, &curr_net->stop_y);
-	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    pick_and_place_reset_bounding_box (curr_net);
-	    
-	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	    curr_net = curr_net->next;
-	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
-	    
+
+	    curr_net = pnp_new_net(curr_net);
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_LINEARx1);
+
 	    gerb_transf_apply(-partData.length/2, -partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
 	    gerb_transf_apply(partData.length/2, -partData.width/2, tr_rot, 
 			      &curr_net->stop_x, &curr_net->stop_y);
-	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    pick_and_place_reset_bounding_box (curr_net);
-	    
-	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	    curr_net = curr_net->next;
-	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
-	    
+
+	    curr_net = pnp_new_net(curr_net);
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_LINEARx1);
+
 	    gerb_transf_apply(partData.length/2, -partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
 	    gerb_transf_apply(partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->stop_x, &curr_net->stop_y);
-	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    pick_and_place_reset_bounding_box (curr_net);
-	    
-	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	    curr_net = curr_net->next;
-	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
-	    
+
+	    curr_net = pnp_new_net(curr_net);
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_LINEARx1);
+
 	    if (partData.shape == PART_SHAPE_RECTANGLE) {
 		gerb_transf_apply(partData.length/4, -partData.width/2, tr_rot, 
 				  &curr_net->start_x, &curr_net->start_y);
@@ -754,35 +705,27 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 				  &curr_net->start_x, &curr_net->start_y);
 		gerb_transf_apply(partData.length/4, partData.width/4, tr_rot, 
 				  &curr_net->stop_x, &curr_net->stop_y);
-		
-		curr_net->aperture = 0;
-		curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-		curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-		curr_net->layer = image->layers;
-		curr_net->state = image->states;
-		pick_and_place_reset_bounding_box (curr_net);
-		
-		curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-		curr_net = curr_net->next;
-		assert(curr_net);
-		memset((void *)curr_net, 0, sizeof(gerbv_net_t));
+
+		curr_net = pnp_new_net(curr_net);
+		pnp_init_net(curr_net, image, partData.designator,
+				GERBV_APERTURE_STATE_ON,
+				GERBV_INTERPOLATION_LINEARx1);
+
 		gerb_transf_apply(partData.length/2, partData.width/4, tr_rot, 
 				  &curr_net->start_x, &curr_net->start_y);
 		gerb_transf_apply(partData.length/4, partData.width/4, tr_rot, 
 				  &curr_net->stop_x, &curr_net->stop_y);     
 	    }
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    pick_and_place_reset_bounding_box (curr_net);
-	    
+
 	    /* calculate a rough radius for the min/max screen calcs later */
-	    radius = max (partData.length/2, partData.width/2);
+	    radius = MAX(partData.length/2, partData.width/2);
 	} else {
 	    gdouble tmp_x,tmp_y;
-	    
+
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_LINEARx1);
+
 	    curr_net->start_x = partData.mid_x;
 	    curr_net->start_y = partData.mid_y;
 	    gerb_transf_apply( partData.pad_x -  partData.mid_x, 
@@ -790,37 +733,25 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 			       
 	    curr_net->stop_x = tmp_x;
 	    curr_net->stop_y = tmp_y;
-	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    
-	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
-	    curr_net = curr_net->next;
-	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
-	    
+
+
+	    curr_net = pnp_new_net(curr_net);
+	    pnp_init_net(curr_net, image, partData.designator,
+			    GERBV_APERTURE_STATE_ON,
+			    GERBV_INTERPOLATION_CW_CIRCULAR);
+
 	    curr_net->start_x = partData.mid_x;
 	    curr_net->start_y = partData.mid_y;
 	    curr_net->stop_x = partData.pad_x;
 	    curr_net->stop_y = partData.pad_y;
-	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
-	    curr_net->interpolation = GERBV_INTERPOLATION_CW_CIRCULAR;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    pick_and_place_reset_bounding_box (curr_net);
-	    
-	    curr_net->cirseg = g_new0 (gerbv_cirseg_t,1);
+
+	    curr_net->cirseg = g_new0 (gerbv_cirseg_t, 1);
 	    curr_net->cirseg->angle1 = 0.0;
 	    curr_net->cirseg->angle2 = 360.0;
 	    curr_net->cirseg->cp_x = partData.mid_x;
 	    curr_net->cirseg->cp_y = partData.mid_y;
-	    radius = sqrt((partData.pad_x-partData.mid_x)*(partData.pad_x-partData.mid_x) +
-			  (partData.pad_y-partData.mid_y)*(partData.pad_y-partData.mid_y));
+	    radius = hypot(partData.pad_x - partData.mid_x,
+			    partData.pad_y-partData.mid_y);
 	    if (radius < 0.001)
 	    	radius = 0.1;
 	    curr_net->cirseg->width = 2*radius; /* fabs(pad_x-mid_x) */
@@ -831,10 +762,10 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 	 * update min and max numbers so the screen zoom-to-fit 
 	 *function will work
 	 */
-	image->info->min_x = min(image->info->min_x, (partData.mid_x - radius - 0.02));
-	image->info->min_y = min(image->info->min_y, (partData.mid_y - radius - 0.02));
-	image->info->max_x = max(image->info->max_x, (partData.mid_x + radius + 0.02));
-	image->info->max_y = max(image->info->max_y, (partData.mid_y + radius + 0.02));
+	image->info->min_x = MIN(image->info->min_x, (partData.mid_x - radius - 0.02));
+	image->info->min_y = MIN(image->info->min_y, (partData.mid_y - radius - 0.02));
+	image->info->max_x = MAX(image->info->max_x, (partData.mid_x + radius + 0.02));
+	image->info->max_y = MAX(image->info->max_y, (partData.mid_y + radius + 0.02));
     }
     curr_net->next = NULL;
     
@@ -847,6 +778,8 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
  *	pick_and_place_parse_file_to_images
  *	------------------------------------------------------------------
  *	Description: Renders a pick and place file to a gerb_image.
+ *	If image pointer is not NULL, then corresponding image will not be
+ *	populated.
  *	Notes: The file format should already be verified before calling
  *       this function, since it does very little sanity checking itself.
  *	------------------------------------------------------------------
@@ -858,10 +791,51 @@ pick_and_place_parse_file_to_images(gerb_file_t *fd, gerbv_image_t **topImage,
 	GArray *parsedPickAndPlaceData = pick_and_place_parse_file (fd);
 
 	if (parsedPickAndPlaceData != NULL) {
-		*bottomImage = pick_and_place_convert_pnp_data_to_image(parsedPickAndPlaceData, 0);
-		*topImage = pick_and_place_convert_pnp_data_to_image(parsedPickAndPlaceData, 1);
+		/* Non NULL pointer is used as "not to reload" mark */
+		if (*bottomImage == NULL)
+			*bottomImage = pick_and_place_convert_pnp_data_to_image(parsedPickAndPlaceData, 0);
+
+		if (*topImage == NULL)
+			*topImage = pick_and_place_convert_pnp_data_to_image(parsedPickAndPlaceData, 1);
 
 		g_array_free (parsedPickAndPlaceData, TRUE);
 	}
 } /* pick_and_place_parse_file_to_images */
 
+static gerbv_net_t *
+pnp_new_net(gerbv_net_t *net)
+{
+	gerbv_net_t *n;
+	net->next = g_new0(gerbv_net_t, 1);
+	n = net->next;
+	assert(n != NULL);
+
+	pnp_reset_bbox (n);
+
+	return n;
+}
+
+static void
+pnp_reset_bbox (gerbv_net_t *net)
+{
+	net->boundingBox.left = -HUGE_VAL;
+	net->boundingBox.right = HUGE_VAL;
+	net->boundingBox.bottom = -HUGE_VAL;
+	net->boundingBox.top = HUGE_VAL;
+}
+
+static void
+pnp_init_net(gerbv_net_t *net, gerbv_image_t *image, const char *label,
+		gerbv_aperture_state_t apert_state,
+		gerbv_interpolation_t interpol)
+{
+	net->aperture = 0;
+	net->aperture_state = apert_state;
+	net->interpolation = interpol;
+	net->layer = image->layers;
+	net->state = image->states;
+
+	if (strlen(label) > 0) {
+		net->label = g_string_new (label);
+	}
+}
