@@ -23,7 +23,7 @@
  */
  
 /** \file project.h
-    \brief routines for loading and saving project files
+    \brief Header info for loading and saving project files.
     \ingroup gerbv
  */ 
 
@@ -34,7 +34,15 @@ typedef struct project_list_t {
     int layerno;
     char *filename;
     int rgb[3];
+    int alpha;
     char inverted;
+    double translate_x;
+    double translate_y;
+    double rotation;
+    double scale_x;
+    double scale_y;
+    char mirror_x;
+    char mirror_y;
     char is_pnp;
     char visible;
     gerbv_HID_Attribute *attr_list;
@@ -48,6 +56,7 @@ enum conv_type {
     UNIX_MINGW = 1
 };
 
+int project_is_gerbv_project(const char *filename, gboolean *ret);
 
 /*
  * Reads a project from a file and returns a linked list describing the project
