@@ -23,8 +23,8 @@
  */
 
 /** \file render.h
-    \brief Header info for the rendering support functions for libgerbv
-    \ingroup libgerbv
+    \brief Header info for the rendering support functions for gerbv
+    \ingroup gerbv
 */
 
 #include "gerber.h"
@@ -57,22 +57,24 @@ render_draw_measure_distance(void);
 void render_refresh_rendered_image_on_screen (void);
 
 void
-render_clear_selection_buffer (void);
-
-void
-render_remove_selected_objects_belonging_to_layer (gint index);
+render_remove_selected_objects_belonging_to_layer (
+			gerbv_selection_info_t *sel_info, gerbv_image_t *image);
 
 void
 render_free_screen_resources (void);
 
-void render_fill_selection_buffer_from_mouse_click (gint mouseX, gint mouseY, gint activeFileIndex,
-		gboolean eraseOldSelection);
-void
-render_fill_selection_buffer_from_mouse_drag (gint corner1X, gint corner1Y,
-	gint corner2X, gint corner2Y, gint activeFileIndex, gboolean eraseOldSelection);
-
-gerbv_render_info_t screenRenderInfo;
+enum selection_action {
+	SELECTION_REPLACE = 0,
+	SELECTION_ADD,
+	SELECTION_TOGGLE,
+};
 
+void render_fill_selection_buffer_from_mouse_click (gint mouseX, gint mouseY,
+		gint activeFileIndex, enum selection_action action);
 
+void render_fill_selection_buffer_from_mouse_drag (gint corner1X, gint corner1Y,
+		gint corner2X, gint corner2Y,
+		gint activeFileIndex, enum selection_action action);
 
+extern gerbv_render_info_t screenRenderInfo;
 
